# Scraper and Back Scraper for New York Commercial Division
# CourtID: nysupct
# Court Short Name: NY

from juriscraper.opinions.united_states.state import nyappterm_1st


class Site(nyappterm_1st.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court = "Commercial Division"
        self.parameters.update({"court": self.court})



"""
Scraper for the Supreme Court of Tennessee
CourtID: tenn
Court Short Name: Tenn.
"""
import time
from datetime import date

from lxml import html

from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.tsc.state.tn.us/courts/supreme-court/opinions"
        self.court_id = self.__module__
        self.back_scrape_iterable = list(range(0, 131))

    def _get_download_urls(self):
        return [
            t
            for t in self.html.xpath(
                "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/a/@href"
            )
        ]

    def _get_case_names(self):
        return [
            t
            for t in self.html.xpath(
                "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/a/text()"
            )
        ]

    def _get_lower_courts(self):
        return [
            t.strip()
            for t in self.html.xpath(
                "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td[2]/text()"
            )
        ]

    def _get_case_dates(self):
        dates = []
        for s in self.html.xpath(
            "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td[4]/span/text()"
        ):
            dates.append(
                date.fromtimestamp(time.mktime(time.strptime(s, "%m/%d/%y")))
            )
        return dates

    def _get_docket_numbers(self):
        return [
            t
            for t in self.html.xpath(
                "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/div[./@class[contains(.,'number')]]/text()"
            )
        ]

    # Here we are using the 'judges' field to list the Authoring Judge.
    # It would be better for AbstractSite to have an 'author' field and to put
    # these there.
    def _get_judges(self):
        judges = []
        for t in self.html.xpath(
            "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/div/text()"
        ):
            if "Authoring" in t:
                # We strip the text 'Authoring Judge: '
                judges.append(t[17:])
            else:
                continue
        return judges

    def _get_lower_court_judges(self):
        trial_judges = []
        for t in self.html.xpath(
            "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td/div/text()"
        ):
            if "Trial" in t:
                # We strip the text 'Trial Court Judge: '
                trial_judges.append(t[19:])
            else:
                continue
        return trial_judges

    def _get_summaries(self):
        summaries = []
        for e in self.html.xpath(
            "//table//tr[not(descendant::a[contains(@href, 'pendingcase')]) and descendant::a/text()]/td//div[@class='case-body']"
        ):
            ps = e.xpath("p")
            s = ""
            for p in ps:
                s += html.tostring(p, method="text", encoding="unicode")
            summaries.append(s)
        return summaries

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _download_backwards(self, page):
        self.url = f"{self.url}?page={page}"
        self.html = self._download()



"""
History:
    - 2014-08-05: Updated by mlr because it was not working, however, in middle
      of update, site appeared to change. At first there were about five
      columns in the table and scraper was failing. Soon, there were seven and
      the scraper started working without my fixing it. Very odd.
    - 2023-01-13: Update to use RSS Feed
"""

import feedparser
from lxml.html import tostring

from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.ca9.uscourts.gov/opinions/index.xml"
        self.status = "Published"

    def _process_html(self) -> None:
        feed = feedparser.parse(tostring(self.html))
        for item in feed["entries"]:
            self.cases.append(
                {
                    "name": titlecase(item["title"]),
                    "url": item["link"],
                    "date": item["published"],
                    "status": self.status,
                    "docket": item["link"].split("/")[-1][:-4],
                }
            )



# Author: Michael Lissner
# Date created: 2013-08-10

from juriscraper.opinions.united_states.state import ark


class Site(ark.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court = "courtofappeals"
        self.url = f"https://opinions.arcourts.gov/ark/{self.court}/en/rss.do"
        self.cite_regex = r"\d{2,4} Ark\. App\. \d+"



"""Scraper for North Carolina Court of Appeals
CourtID: ncctapp
Court Short Name: N.C. Ct. App.
Author: Jon Andersen
History:
    2014-08-04: Created by Jon Andersen
"""

from datetime import date

from juriscraper.opinions.united_states.state import nc


class Site(nc.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "http://appellate.nccourts.org/opinions/?c=coa&year=%s"
            % date.today().year
        )
        self.back_scrape_iterable = list(
            range((date.today().year - 1), 1997, -1)
        )

    def _download_backwards(self, year):
        self.url = f"http://appellate.nccourts.org/opinions/?c=coa&year={year}"
        self.html = self._download()



"""Scraper for the Supreme Court of Delaware
CourtID: del

Creator: Andrei Chelaru
Reviewer: mlr
"""

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = (
            "http://courts.delaware.gov/opinions/list.aspx?ag=supreme%20court"
        )
        # Note that we can't do the usual thing here because 'del' is a Python keyword.
        self.court_id = "juriscraper.opinions.united_states.state.del"

    def _get_case_dates(self):
        return [
            convert_date_string(date.strip())
            for date in self._get_nth_cell_data(2, text=True)
        ]

    def _get_download_urls(self):
        return [url.strip() for url in self._get_nth_cell_data(1, href=True)]

    def _get_case_names(self):
        return [
            name.strip() for name in self._get_nth_cell_data(1, link_text=True)
        ]

    def _get_docket_numbers(self):
        return [
            docket.strip()
            for docket in self._get_nth_cell_data(3, link_text=True)
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _get_judges(self):
        # We need special logic here because they use <br> tags in the cell text
        return [
            " ".join(cell.xpath("text()"))
            for cell in self.html.xpath("//table/tr/td[6]")
        ]

    def _get_nth_cell_data(
        self, cell_number, text=False, href=False, link_text=False
    ):
        # Retrieve specific types of data from all nTH cells in the table
        path = "//table/tr/td[%d]" % cell_number
        if text:
            path += "/text()"
        elif href:
            path += "/a/@href"
        elif link_text:
            path += "/a/text()"
        return self.html.xpath(path)



"""
Scraper for the United States Bankruptcy Appellate Panel for the Tenth Circuit
CourtID: bap10
Court Short Name: 10th Cir. BAP
Auth: Jon Andersen <janderse@gmail.com>
Reviewer: mlr
History:
    2014-09-01: First draft by Jon Andersen
    2014-09-02: Revised by mlr to use _clean_text() instead of pushing logic
                into the _get_case_dates function.
"""

from datetime import date, timedelta
from urllib.parse import urlencode

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.bap10.uscourts.gov/opinion/search/results"
        self.court_id = self.__module__
        today = date.today()
        params = urlencode(
            {
                "keywords": "",
                "parties": "",
                "judges": "",
                "field_opinion_date_value[min][date]": (
                    today - timedelta(30)
                ).strftime("%m/%d/%Y"),
                "field_opinion_date_value[max][date]": today.strftime(
                    "%m/%d/%Y"
                ),
                "exclude": "",
            }
        )
        self.url = (
            f"https://www.bap10.uscourts.gov/opinion/search/results?{params}"
        )

    def _process_html(self):
        for row in self.html.xpath(".//tr"):
            if not row.xpath(".//td"):
                continue
            self.cases.append(
                {
                    "docket": row.xpath(".//td")[0].text_content().strip(),
                    "name": row.xpath(".//td")[1].text_content().strip(),
                    "date": row.xpath(".//td")[2].text_content().strip(),
                    "url": row.xpath(".//a/@href")[0],
                    "status": "Published",
                }
            )



"""Scraper for the Michigan Court of Appeals
CourtID: michctapp
Court Short Name: Mich. Ct. App.
Type: Published and Unpublished
Reviewer: mlr
History:
    - 2014-09-19: Created by Jon Andersen
    - 2022-01-28: Updated for new web site, @satsuki-chan.
"""
from typing import List
from urllib.parse import urlencode

from juriscraper.opinions.united_states.state import mich


class Site(mich.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court = "Court Of Appeals"
        params = self.filters + (("aAppellateCourt", self.court),)
        self.url = f"https://www.courts.michigan.gov/api/CaseSearch/SearchCaseOpinions?{urlencode(params)}"

    def _get_precedential_statuses(self) -> List[str]:
        """Find Precedential Status

        If the case is published they note Published in the title string.

        :return: Precedential statuses
        """
        for case in self.cases:
            case["precedential_status"] = self.get_status(case["title"])
        return [case["precedential_status"] for case in self.cases]

    def get_status(self, title: str) -> str:
        """Get the status of a case

        :param title: The JSON API title string
        :return: The status of the case
        """
        if "Published" in title:
            status = "Published"
        else:
            status = "Unpublished"
        return status



"""
Scraper for Massachusetts Supreme Court
CourtID: mass
Court Short Name: MS
Author: Andrei Chelaru
Court Contact: SJCReporter@sjc.state.ma.us (617) 557-1030
Reviewer: mlr
History:
 - 2014-07-12: Created.
 - 2014-08-05, mlr: Updated regex.
 - 2014-09-18, mlr: Updated regex.
 - 2016-09-19, arderyp: Updated regex.
 - 2017-11-29, arderyp: Moved from RSS source to HTML
    parsing due to website redesign
 - 2023-01-28, William Palin: Updated scraper
"""

import datetime
import re
from datetime import datetime

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.mass.gov/service-details/new-opinions"
        self.court_id = self.__module__
        self.court_identifier = "SJC"

    def _process_html(self):
        for file in self.html.xpath(".//a/@href[contains(.,'pdf')]/.."):
            content = file.text_content()
            m = re.search(r"(.*?) \((.*?)\)( \((.*?)\))?", content)
            name, docket, _, date = m.groups()
            if self.court_identifier not in docket:
                continue
            url = file.get("href")
            parts = url.split("/")[-4:-1]
            parts = [int(d) for d in parts]
            date = datetime(year=parts[0], month=parts[1], day=parts[2]).date()
            self.cases.append(
                {
                    "name": name,
                    "status": "Published",
                    "date": str(date),
                    "docket": docket,
                    "url": url,
                }
            )



import pprint
import re
import sys
from typing import Dict, Optional

from lxml import html

from juriscraper.lib.html_utils import strip_bad_html_tags_insecure
from juriscraper.lib.string_utils import force_unicode
from juriscraper.pacer.utils import get_pacer_doc_id_from_doc1_url

from ..lib.log_tools import make_default_logger
from .reports import BaseReport

logger = make_default_logger()


class AppellateAttachmentPage(BaseReport):
    """An object for querying and parsing the appellate att. page report.

    * Some notes on Appellate attachement pages.
    The document number for the main document is not identified as such in the
    HTML.  Trial and error suggests it is the lowest numbered document
    number.  This is not always the first document listed in the order of
    attachments on the appellate attachment page.
    """

    PATH = "docs1/"

    def __init__(self, court_id, pacer_session=None):
        super().__init__(court_id, pacer_session)

    def query(self, document_number):
        """Query the "attachment page" endpoint and set the results to self.response.

        The appellate attachment page uses the a variable called dls_id
        in the URL path which appears to be a variation of the main document
        id number.  We can generate the DLS ID by changing the fourth number
        in the document ID to 0.
        :param document_number: The internal PACER document ID for the item.
        :return: a request response object
        """
        assert (
            self.session is not None
        ), "session attribute of DocketReport cannot be None."

        # Generate the document URL from the document number.
        document_number = f"{document_number[:3]}0{document_number[4:]}"
        url = self.url + document_number
        logger.info("Querying the attachment page endpoint at URL: %s", url)
        self.response = self.session.get(url)
        self.parse()

    def _strip_bad_html_tags_insecure(self, text: str) -> None:
        """Override base function to include scripts."""
        self.tree = strip_bad_html_tags_insecure(text, remove_scripts=False)

    @property
    def data(self) -> Dict:
        """Get data back from the query for the matching document entry.

        Appellate attachments is different than the district court.  We have
        no document_id. Instead we can use the pacer_seq_no or the dls_id
        to discern which attachment goes with which docket entry.

        Additionally, the main document is not identifiable in attachments.
        It is presumed to be the document with the lowest pacer_doc_id.

        :return: If lookup fails, an empty dict. Else, a dict containing the
        following fields:
            - pacer_doc_id: The id of the main document.
            - pacer_case_id: The pacer case id for this case.
            - pacer_seq_no - The value of the docket row we came from.

            - attachments: A list of attached items with the following fields:
                - attachment_number: The attachment number.
                - description: A description of the item.
                - page_count: The number of pages for the attachment.
                - pacer_doc_id: The pacer doc id for the attachment (a str).

        See the JSON objects in the tests for more examples.
        """
        if self.is_valid is False:
            return {}

        # Find the table/tr/td containing a link and return the tr's
        rows = self.tree.xpath("//tbody/tr/td/a/parent::td/parent::tr")
        if not rows:
            logger.info("No documents found on attachment page.")
            return {}

        result = {
            "pacer_doc_id": self._get_main_pacer_doc_id(),
            "pacer_case_id": self._get_pacer_case_id(),
            "pacer_seq_no": self._get_pacer_seq_no(),
            "attachments": [],
        }

        for row in rows:
            result["attachments"].append(
                {
                    "attachment_number": self._get_attachment_number(row),
                    "description": self._get_description_from_tr(row),
                    "page_count": self._get_page_count_from_tr(row),
                    "pacer_doc_id": self._get_pacer_doc_id(row),
                }
            )
        return result

    def _get_main_pacer_doc_id(self):
        """Extract the main pacer_doc_id.

        The DLS ID  This should be used to identify
        document number row.

        In appellate attachment page, this is easy to extract with an XPath.
        :return: main pacer_doc_id
        """
        try:
            rows = self.tree.xpath("//tbody/tr/td/a/@href")
            pacer_doc_ids = []
            for url_row in rows:
                pacer_doc_ids.append(get_pacer_doc_id_from_doc1_url(url_row))
            return sorted(pacer_doc_ids)[0]
        except IndexError:
            return None

    def _get_attachment_number(self, row: html.HtmlElement) -> int:
        """Return the attachment number for an item.

        :param row: Table row as an lxml element
        :return: Attachment number for row
        """
        number = int(row.xpath(".//td/text()")[0].strip())
        return number

    def _get_description_from_tr(self, row: html.HtmlElement) -> str:
        """Get the description from the row

        :param row: Table row
        :return: Attachment description
        """
        row_nodes = row.xpath(".//td")
        if not row_nodes:
            return ""
        description = row_nodes[-2].xpath("text()")
        if description:
            return force_unicode(description[0].strip())
        return ""

    @staticmethod
    def _get_page_count_from_tr(row: html.HtmlElement) -> Optional[int]:
        """Take a row from the attachment table and return the page count as an
        int extracted from the cell specified by index.

        :param row: Table row as an lxml element
        :return: Attachment page count
        """
        description_text_nodes = row.xpath(".//td/text()")
        if not description_text_nodes:
            return None
        return int(description_text_nodes[-1].strip())

    @staticmethod
    def _get_pacer_doc_id(row: html.HtmlElement) -> Optional[str]:
        """Take in a row from the attachment table and return the pacer_doc_id
        for the item in that row. Return None if the ID cannot be found.

        :param row: Table row as an lxml element
        :return: Attachment pacer_doc_id
        """
        try:
            url = row.xpath(".//a")[0]
        except IndexError:
            # Item exists, but cannot download document. Perhaps it's sealed
            # or otherwise unavailable in PACER. This is carried over from the
            # docket report and may not be needed here, but it's a good
            # precaution.
            return None
        else:
            doc1_url = url.xpath("./@href")[0]
            return get_pacer_doc_id_from_doc1_url(doc1_url)

    def _get_pacer_case_id(self) -> str:
        """Get the pacer_case_id value by inspecting the function scripts

        :returns str: The pacer_case_id value
        """
        script_html = html.tostring(self.tree.xpath(".//script")[0]).decode()
        m = re.search(r"[?&]caseid=(\d+)", script_html, flags=re.I)
        if m:
            return m.group(1)

    def _get_pacer_seq_no(self) -> str:
        """Get pacer sequence number.
        This value corresponds to the value in the docket TR

        :return: The pacer_seq_no
        """
        script_html = html.tostring(self.tree.xpath(".//script")[0]).decode()
        m = re.search(r"[?&]d=(\d+)&outputForm", script_html, flags=re.I)
        if m:
            return m.group(1)


def main():
    if len(sys.argv) != 2:
        print(
            "Usage: python -m juriscraper.pacer.appellate_attachment_page filepath"
        )
        print("Please provide a path to an HTML file to parse.")
        sys.exit(1)
    report = AppellateAttachmentPage(
        "cand"
    )  # Court ID is only needed for querying.
    filepath = sys.argv[1]
    print(f"Parsing HTML file at {filepath}")
    with open(filepath) as f:
        text = f.read()
    report._parse_text(text)
    pprint.pprint(report.data, indent=2)


if __name__ == "__main__":
    main()



"""Scraper for Army Court of Criminal Appeals
CourtID: acca
Reviewer: None
History:
  2015-01-08: Created by mlr
  2016-03-17: Website appears to be dead. Scraper disabled in __init__.py.
  2019-09-11: Rewritten by arderyp to user Linear pattern
"""

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.jagcnet.army.mil/85257546006DF36B/ODD?OpenView&Count=-1"
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath('//tr[@class="domino-viewentry"]'):
            cell_three = row.xpath(".//td[3]")[0]
            text = cell_three.text_content()
            text_parts = text.split("-", 1)
            hrefs = cell_three.xpath(".//a/@href")

            # skip rows without link to opinion
            if not hrefs:
                continue

            self.cases.append(
                {
                    "date": row.xpath(".//td[1]")[0].text_content(),
                    "docket": text_parts[0],
                    "name": text_parts[1],
                    "url": hrefs[0],
                }
            )



"""Scraper for Armed Services Board of Contract Appeals
CourtID: asbca
Court Short Name: ASBCA
Author: Jon Andersen
Reviewer: mlr
History:
    2014-09-11: Created by Jon Andersen
    2016-03-17: Website and phone are dead. Scraper disabled in __init__.py.
"""

import re
from datetime import datetime

from juriscraper.lib.string_utils import clean_if_py3, convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "http://www.asbca.mil/Decisions/decisions%d.html"
            % datetime.today().year
        )
        self.columns = None
        self.back_scrape_iterable = list(range(2013, 2000 - 1, -1))

    # Fix broken month names and funky whitespace usage.
    def _clean_text(self, text):
        text = super()._clean_text(text)
        text = text.replace("&#160;", " ").replace("&nbsp;", " ")
        text = text.replace("Januray", "January")
        text = text.replace("Februrary", "February")
        text = re.sub(re.compile(r"[\s]+", flags=re.MULTILINE), " ", text)
        return text

    def parse_column_names(self):
        # Lookup column names and save them for later
        self.columns = dict()
        path = "//table/tr[1]/td"
        i = 1
        for column in self.html.xpath(path):
            colname = clean_if_py3("".join(column.itertext())).strip()
            self.columns[colname] = i
            i += 1
        return self.columns

    def _get_case_dates(self):
        self.parse_column_names()
        path = "//table/tr[td/a]/td[%d]/text()" % (
            self.columns["Decision Date"]
        )
        return [
            self._get_date_object_from_string(date_string)
            for date_string in self.html.xpath(path)
        ]

    def _get_date_object_from_string(self, date_string):
        date_string = (
            clean_if_py3(date_string)
            .strip()
            .replace(" ,", ", ")
            .replace("2104", "2014")
        )
        return convert_date_string(date_string)

    def _get_case_names(self):
        path = "//table/tr/td/a[1]"
        case_names = [
            clean_if_py3("".join(txt.itertext()).strip())
            for txt in self.html.xpath(path)
        ]
        return case_names

    def _get_download_urls(self):
        path = "//table/tr/td/a[1]/@href"
        return [clean_if_py3(href).strip() for href in self.html.xpath(path)]

    def _get_judges(self):
        path = "//table/tr[td/a]/td[%d]/text()" % (self.columns["Judge"],)
        return [clean_if_py3(txt).strip() for txt in self.html.xpath(path)]

    def _get_docket_numbers(self):
        if "ASBCA Number" not in self.columns:
            return None
        path = "//table/tr[td/a]/td[%d]/text()" % self.columns["ASBCA Number"]
        return [
            f"ASBCA No. {clean_if_py3(txt).strip()}"
            for txt in self.html.xpath(path)
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _download_backwards(self, year):
        self.url = "http://www.asbca.mil/Decisions/decisions%d.html" % year
        if year == 2010:
            self.url = "http://www.asbca.mil/Decisions/decisions.html"
        self.html = self._download()

    def _get_case_name_shorts(self):
        # We don't (yet) support short case names for administrative bodies.
        return None



from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://nvcourts.gov/Supreme/Decisions/Advance_Opinions/"
        self.status = "Published"

        # HTTPS certificate is bad, but hopefully they'll fix it and we can remove the line below
        self.disable_certificate_verification()

    def _process_html(self):
        table_row_path = '//table[@style="border-spacing:0px"]//tr'
        for row in self.html.xpath(table_row_path):
            citation = row.xpath(".//td[1]")[0].text_content().strip()
            docket = row.xpath(".//td[2]")[0].text_content()
            name = row.xpath(".//td[3]")[0].text_content()
            url = row.xpath(".//td[4]//a/@href")
            date_string = row.xpath(".//td[4]")[0].text_content()
            year = convert_date_string(date_string).year

            self.cases.append(
                {
                    "date": date_string,
                    "docket": docket,
                    "name": name if name else "Unknown",
                    "citation": "%d NV %s" % (year, citation),
                    "url": url[0],
                }
            )



"""
Scraper for Florida 3rd District Court of Appeal
CourtID: flaapp3
Court Short Name: flaapp3
Contact: 3dca@flcourts.org, Angel Falero <faleroa@flcourts.org> (305-229-6743)
"""

from juriscraper.lib.html_utils import (
    get_table_column_links,
    get_table_column_text,
)
from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        # Browser url: https://www.3dca.flcourts.org/Opinions/Most-Recent-Opinion-Release
        self.url = "https://www.3dca.flcourts.org/search/opinions/?sort=opinion/type%20desc,%20opinion/case_number%20asc&view=embed_custom&searchtype=opinions&recent_only=1&hide_search=1&hide_filters=1&limit=75&offset=0"

    def _get_case_names(self):
        return get_table_column_text(self.html, 4)

    def _get_download_urls(self):
        return get_table_column_links(self.html, 1)

    def _get_case_dates(self):
        date_strings = get_table_column_text(self.html, 7)
        return [convert_date_string(ds) for ds in date_strings]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _get_docket_numbers(self):
        return get_table_column_text(self.html, 3)



# Author: Michael Lissner
# History:
# - 2013-06-11: Birth.
# - 2013-08-06: Revised by Brian Carver
# - 2014-08-05: Updated URL by mlr
# - 2022-01-01: Updated by flooie

import datetime
import re
from typing import Any, Dict

from juriscraper.lib.html_utils import get_row_column_text
from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Published"
        self.url = "https://ujs.sd.gov/Supreme_Court/opinions.aspx"
        self.years = range(2010, datetime.date.today().year)

    def _set_page_year(
        self,
        year: int,
        view_state: str,
        event_validation: str,
    ) -> None:
        a = self.html.xpath(
            f".//a[contains(@id,'ContentPlaceHolder1_ChildContent1_Repeater_OpinionsYear_LinkButton1')][contains(text(), '{year}')]"
        )
        link = a[0].get("href").split("'")[1]
        data = {
            "__VIEWSTATE": view_state,
            "__EVENTVALIDATION": event_validation,
            "__EVENTTARGET": link,
        }
        self.parameters = data
        self.method = "POST"

    def _set_page_next(
        self,
        view_state: str,
        event_validation: str,
    ) -> None:
        data = {
            "ctl00$ctl00$ScriptManager1": "ctl00$ctl00$ContentPlaceHolder1$ChildContent1$UpdatePanel_Opinions|ctl00$ctl00$ContentPlaceHolder1$ChildContent1$LinkButton_Next",
            "__EVENTTARGET": "ctl00$ctl00$ContentPlaceHolder1$ChildContent1$LinkButton_Next",
            "__VIEWSTATE": view_state,
            "__EVENTVALIDATION": event_validation,
        }
        self.parameters = data
        self.method = "POST"

    def _process_html(self):
        for year in self.years:
            view_state = self.html.xpath("//input[@id='__VIEWSTATE']")[0].get(
                "value"
            )
            event_validation = self.html.xpath(
                "//input[@id='__EVENTVALIDATION']"
            )[0].get("value")

            self._set_page_year(year, view_state, event_validation)
            self.html = super()._download()
            while True:
                rows = self.html.xpath(
                    "//div[@id='ContentPlaceHolder1_ChildContent1_UpdatePanel_Opinions']//tbody/tr"
                )
                for row in rows:
                    cite = re.findall(
                        r"\d{4} S\.D\. \d+", get_row_column_text(row, 2)
                    )
                    if not cite:
                        continue

                    # https://ujs.sd.gov/uploads/sc/opinions/2928369ef9a6.pdf
                    # We abstract out the first part of the docket number here
                    # And process the full docket number in the `extract_from_text` method
                    # Called after the file has been downloaded.
                    docket = row.xpath(".//td[2]/a/@href")[0].split("/")[-1][
                        :5
                    ]
                    self.cases.append(
                        {
                            "date": get_row_column_text(row, 1),
                            "name": titlecase(
                                get_row_column_text(row, 2).rsplit(",", 1)[0]
                            ),
                            "neutral_citation": cite[0],
                            "url": row.xpath(".//td[2]/a/@href")[0],
                            "docket": docket,
                        }
                    )
                page_count = self.html.xpath(
                    "//span[@id='ContentPlaceHolder1_ChildContent1_Label_Page']"
                )[0].text_content()
                page_of = re.findall(r"Page (\d+) of (\d+)", page_count)
                if len(set(page_of[0])) == 1:
                    break

                vs = self.html.xpath("//input[@id='__VIEWSTATE']")[0].get(
                    "value"
                )
                ev = self.html.xpath("//input[@id='__EVENTVALIDATION']")[
                    0
                ].get("value")

                self._set_page_next(view_state=vs, event_validation=ev)
                self.html = super()._download()

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Can we extract the date filed from the text?

        :param scraped_text: The content of the document downloaded
        :return: Metadata to be added to the case
        """
        # The docket number appears to be the first text on the page.
        # So I crop the text to avoid any confusion that might occur in the
        # body of an opinion.
        docket = re.findall(r"#\d+.*-.-\w{3}", scraped_text)[0][:20]
        metadata = {
            "Docket": {
                "docket_number": docket,
            },
        }
        return metadata



"""Scraper for Colorado Appeals Court
CourtID: coloctapp
Court Short Name: Colo. Ct. App.

History:
    - 2022-01-31: Updated by William E. Palin
    - 2023-01-05: Updated by William E. Palin
"""


import datetime
import re

from lxml.html import fromstring

from juriscraper.opinions.united_states.state import colo


class Site(colo.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.month = datetime.date.today().month
        self.year = datetime.date.today().year
        self.url = f"https://www.courts.state.co.us/Courts/Court_of_Appeals/Case_Announcements/Index.cfm?year={self.year}&month={self.month}&Submit=Go"

    def _process_html(self):
        if not self.test_mode_enabled():
            self.initiate_webdriven_session()

        if not self.test_mode_enabled():
            links = self.html.xpath(".//a[@class='touchable-link']")
        else:
            links = ["fake_link"]

        for link in links:
            if self.test_mode_enabled():
                super()._download()
                date_filed = "December 29, 2022"
            else:
                date_filed = link.text_content()
                self.webdriver.get(link.get("href"))
                # Wait for pdf to render
                self.webdriver.implicitly_wait(10000)
                # And click next twice to make sure we fully render the PDF content
                self.find_element_by_id("next").click()
                self.find_element_by_id("next").click()
                self.html = fromstring(self.webdriver.page_source)

            pattern = r"(2022COA\d+)\n(.*?)\n(Division \w+)"
            urls_to_add = []
            for pg in self.html.xpath(".//div[@class='page']"):
                if not pg.xpath(".//span[@role='link']/span/@aria-owns"):
                    continue
                anno_ids = pg.xpath(".//span[@role='link']/span/@aria-owns")
                for anno_id in anno_ids:
                    urls = pg.xpath(
                        f"//a[@id='pdfjs_internal_id_{anno_id}']/@href"
                    )
                    if urls:
                        url = urls[0]
                        if url not in urls_to_add:
                            urls_to_add.append(url)
                pg_content = []
                for element in pg.xpath(".//span"):
                    if element.get("class") != "markedContent":
                        continue
                    pg_content.append(element.text_content())
                pg_text = "\n".join(pg_content)
                for match in re.finditer(pattern, pg_text, re.DOTALL):
                    citation = match.groups()[0]
                    case_name = self.get_case_name(match.groups()[1])
                    url = urls_to_add.pop(0)
                    docket = url.split("/")[-1][:-4]
                    self.cases.append(
                        {
                            "date": date_filed,
                            "name": case_name,
                            "docket": docket,
                            "status": "Published",
                            "url": url,
                            "citation": citation,
                        }
                    )

        if not self.test_mode_enabled():
            self.webdriver.quit()

    def get_case_name(self, page_content) -> str:
        """Get case name from page content

        :param pg: List of text
        :return: Cleaned case name
        """
        start = False
        content = []
        for row in page_content.split("\n"):
            if not row.strip():
                continue
            no_go_words = [
                "Plaintiff",
                "Appellee",
                "Defendant",
                "Intervenor",
                "Appellant",
            ]
            if any(word in row for word in no_go_words):
                continue
            m = re.findall(r"Honorable .*, Judge", row)
            if m:
                start = True
                continue
            m = re.findall(r"[A-Z ,]{5,}", row)
            if m and m[0] == row:
                start = False
            if not start:
                continue
            content.append(row.strip(","))
        case_name = " ".join(content)
        return case_name



"""
Scraper for Illinois Supreme Court
CourtID: ill
Contact: webmaster@illinoiscourts.gov, 217-558-4490, 312-793-3250
History:
  2013-08-16: Created by Krist Jin
  2014-12-02: Updated by Mike Lissner to remove the summaries code.
  2016-02-26: Updated by arderyp: simplified thanks to new id attribute identifying decisions table
  2016-03-27: Updated by arderyp: fixed to handled non-standard formatting
  2021-11-02: Updated by satsuki-chan: Updated to new page design.
  2022-01-21: Updated by satsuki-chan: Added validation when citation is missing.
"""

import re

from juriscraper.AbstractSite import logger
from juriscraper.lib.html_utils import (
    get_row_column_links,
    get_row_column_text,
)
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.docket_re = r"\d{4} IL (?P<docket>\d+)"
        self.url = (
            f"https://www.illinoiscourts.gov/top-level-opinions?type=supreme"
        )
        self.status = "Published"

    def _get_docket(self, match):
        return match.group("docket")

    def _get_status(self, citation):
        if "-U" in citation:
            return "Unpublished"
        return "Published"

    def _process_html(self) -> None:
        """Process HTML

        Iterate over each table row.
        If a table row does not have a link - skip it and assume
        the opinion has been withdrawn.

        Return: None
        """
        rows = self.html.xpath("//table[@id='ctl04_gvDecisions']/tr")[1:]
        for row in rows:
            # Don't parse rows for pagination, headers, footers or announcements
            if len(row.xpath(".//td")) != 7 or row.xpath(".//table"):
                continue
            name = get_row_column_text(row, 1)
            citation = get_row_column_text(row, 2)
            date = get_row_column_text(row, 3)
            match = re.search(self.docket_re, citation)
            try:
                url = get_row_column_links(row, 1)
            except IndexError:
                logger.warning(
                    f"Opinion '{citation}' has no URL. (Likely a withdrawn opinion)."
                )
                continue

            if not match:
                logger.warning(f"Opinion '{citation}' has no docket.")
                continue

            docket = self._get_docket(match)
            status = self._get_status(citation)

            self.cases.append(
                {
                    "date": date,
                    "name": name,
                    "citation": citation,
                    "url": url,
                    "docket": docket,
                    "status": status,
                }
            )



import email
import re
from datetime import date
from typing import Dict, List, Optional, TypedDict, Union

from lxml.html import HtmlElement

from ..lib.string_utils import clean_string, convert_date_string, harmonize
from .docket_report import BaseDocketReport
from .reports import BaseReport
from .utils import (
    get_pacer_case_id_from_doc1_url,
    get_pacer_case_id_from_nonce_url,
    get_pacer_doc_id_from_doc1_url,
    get_pacer_magic_num_from_doc1_url,
    get_pacer_seq_no_from_doc1_url,
)


class DocketEntryType(TypedDict):
    date_filed: date
    description: str
    document_number: Optional[str]
    document_url: Optional[str]
    pacer_case_id: Optional[str]
    pacer_doc_id: Optional[str]
    pacer_magic_num: Optional[str]
    pacer_seq_no: Optional[str]


class DocketType(TypedDict):
    case_name: str
    date_filed: date
    docket_entries: List[DocketEntryType]
    docket_number: str


class NotificationEmail(BaseDocketReport, BaseReport):
    """A BaseDocketReport for parsing PACER notification email parsing"""

    ERROR_STRINGS = [
        "Notice of Electronic Claims Filing",
        "This is an announcement e-mail message generated by Court action through the CM/ECF system.",
        "Join ZoomGov Meeting",
        "Modified Dkt text from",
        "Modified Date Filed from",
        "Added Correct PDF to document",
    ]

    def __init__(self, court_id):
        self.court_id = court_id
        self.content_type = None
        self.appellate = None
        self.image_attached = False
        self.docket_numbers = []
        self.subject = None
        self.case_names = []
        if self.court_id.endswith("b"):
            self.is_bankruptcy = True
        else:
            self.is_bankruptcy = False
        super().__init__(court_id)

    @property
    def data(self):
        # Emails with attached images should be ignored.
        if self.is_valid is False or self.tree is None or self.image_attached:
            return {}
        base = {
            "court_id": self.court_id,
        }
        if self.content_type == "text/plain":
            parsed = {
                "appellate": self._is_appellate(),
                "contains_attachments": self._contains_attachments_plain(),
                "dockets": self._get_dockets(),
                "email_recipients": self._get_email_recipients_plain(),
            }
        else:
            parsed = {
                "appellate": self._is_appellate(),
                "contains_attachments": self._contains_attachments(),
                "dockets": self._get_dockets(),
                "email_recipients": self._get_email_recipients(),
            }
        return {**base, **parsed}

    def _is_appellate(self) -> bool:
        """Gets the email notice type from the email text.

        :returns: True if is an NDA otherwise False
        """
        if self.appellate is None:
            if "Notice of Docket Activity" in self.tree.text_content():
                self.appellate = True
                return self.appellate
            self.appellate = False
            return self.appellate
        return self.appellate

    def _sibling_path(self, label):
        """Gets the path string for the sibling of a label cell (td)

        Many data attributes are presented in the format of key: value in table syntax
        this is a good way to get the next element over from a label

        :param label: The cell label
        :returns: The Xpath to the next cell
        """
        return f'.//td[contains(., "{label}:")]/following-sibling::td[1]'

    def _get_case_name(self, current_node: HtmlElement) -> str:
        """Gets a cleaned case name from the email text

        :param  current_node: The relative lxml.HtmlElement
        :returns: Case name, cleaned and harmonized
        """
        path = self._sibling_path("Case Name")
        case_name = self._xpath_text_0(current_node, path)
        if not case_name:
            case_name = self._xpath_text_0(current_node, f"{path}/p")
            if not case_name:
                case_name = "Unknown Case Title"

        # Cache case_name before harmonizing it. For its use parsing the
        # short_description.
        self.case_names.append(clean_string(case_name))
        return clean_string(harmonize(case_name))

    def _get_case_name_plain(self) -> str:
        """Gets a cleaned case name from the plain email text

        Raise an exception if a potential multi-docket text/plain notification
        is found so we can get the example and add support for it.

        :returns: Case name, cleaned and harmonized
        """
        email_body = self.tree.text_content()
        regex = r"Case Name:(.*)"
        find_case = re.findall(regex, email_body)
        if len(find_case) > 1:
            raise NotImplementedError(
                f"Received a potential multi-docket text/plain notification. "
                f"This is probably our chance to add support for it. "
                f"court: {self.court_id}"
            )
        if find_case:
            case_name = find_case[0]
        else:
            case_name = "Unknown Case Title"

        # Cache case_name before harmonizing it. For its use parsing the
        # short_description.
        self.case_names.append(clean_string(case_name))
        return clean_string(harmonize(case_name))

    def _get_docket_number(self, current_node: HtmlElement) -> str:
        """Gets a docket number from the email text

        :param  current_node: The relative lxml.HtmlElement
        :returns: Docket number, parsed
        """

        if self._is_appellate():
            path = self._sibling_path("Case Number")
            case_number = self._xpath_text_0(current_node, f"{path}/a")
            return case_number

        path = self._sibling_path("Case Number")
        docket_number = self._parse_docket_number_strs(
            current_node.xpath(f"{path}/a/text()")
        )
        if not docket_number:
            docket_number = self._parse_docket_number_strs(
                current_node.xpath(f"{path}/p/a/text()")
            )
        return docket_number

    def _get_docket_number_plain(self) -> str:
        """Gets a docket number from the plain email text

        :returns: Docket number, parsed
        """
        email_body = self.tree.text_content()
        regex = r"Case Number:(.*)"
        docket_number = re.findall(regex, email_body)
        return self._parse_docket_number_strs(docket_number)

    def _get_date_filed(self) -> date:
        """Gets the filing date from the email text

        :returns: Date filed as date object
        """
        date_filed = re.search(
            r"filed\son\s([\d|\/]*)", clean_string(self.tree.text_content())
        )
        return convert_date_string(
            date_filed[0].lower().replace("filed on ", "")
        )

    def _get_document_number(self, current_node: HtmlElement) -> str:
        """Gets the specific document number the notification is referring to

        :param  current_node: The relative lxml.HtmlElement
        :returns: Document number, cleaned
        """
        path = self._sibling_path("Document Number")
        node = current_node.xpath(path)[0].text_content()
        text_number = clean_string(node)
        if text_number == "No document attached" or text_number == "":
            return None
        words = re.split(r"\(|\s", text_number)
        return words[0]

    def _get_document_number_plain(self) -> str:
        """Gets the specific document number the notification is referring to

        :returns: Document number, cleaned
        """
        email_body = self.tree.text_content()
        regex = r"Document Number:(.*)"
        document_number = re.findall(regex, email_body)
        if document_number:
            return clean_string(document_number[0])
        else:
            return None

    def _get_doc1_anchor(self, current_node: HtmlElement) -> str:
        """Safely retrieves the anchor tag for the document

        :param  current_node: The relative lxml.HtmlElement
        :returns: Anchor tag, if it's found
        """
        try:
            if self._is_appellate():
                path = f"{self._sibling_path('Document(s)')}//a"
            else:
                path = f"{self._sibling_path('Document Number')}//a"
            return current_node.xpath(path)[0]
        except IndexError:
            return None

    def _get_case_anchor(self, current_node: HtmlElement) -> Optional[str]:
        """Safely retrieves the anchor tag for a case.

        :param current_node: The relative lxml.HtmlElement
        :returns: Case anchor tag, if it's found
        """
        try:
            path = f"{self._sibling_path('Case Number')}//a"
            return current_node.xpath(path)[0].xpath("./@href")[0]
        except IndexError:
            return None

    def _get_case_id_from_case_url(self, case_url) -> Optional[str]:
        """Extract the caseid from the case anchor.

        :param case_url: The case_url where to look for the case_id.
        :returns: caseID, if it's found
        """

        if self.appellate:
            match = re.search(r"caseId=(\d+)", case_url)
            if match:
                return match.group(1)
        else:
            return get_pacer_case_id_from_nonce_url(case_url)
        return None

    def _get_description(self, current_node: HtmlElement) -> str:
        """Gets the docket text

        :param  current_node: The relative lxml.HtmlElement
        :returns: Cleaned docket text
        """

        description = ""
        # Paths to look for NEFs description
        main_path = (
            './following::strong[contains(., "Docket Text:")][1]/parent::p/'
        )
        possible_paths = [
            "font[1]/b//text()",
            "b[1]/span//text()",
            "text()",
            "following::font[@face='arial,helvetica']//text()",
        ]

        if self._is_appellate():
            # Paths to look for NDAs description
            main_path = './following::strong[contains(., "Docket Text:")][1]/following::'
            possible_paths = ["text()"]
        for path in possible_paths:
            node = current_node.xpath(f"{main_path}{path}")
            if len(node):
                for des_part in node:
                    if self._is_appellate():
                        if (
                            des_part
                            == "Notice will be electronically mailed to:"
                        ):
                            break
                    description = description + des_part
                description = clean_string(description)
                if description:
                    return description

        raise Exception(
            f"Can't get docket entry description, court: {self.court_id}"
        )

    def _get_description_plain(self):
        """Gets the docket text for plain email

        :returns: Cleaned docket text
        """
        email_body = self.tree.text_content()
        regex = r"^.*?Docket Text:(.*?)electronically mailed to:"
        find_description = re.findall(regex, email_body, re.DOTALL)
        description = ""
        if find_description:
            splitlines = find_description[0].splitlines()
            for index_line in range(len(splitlines)):
                if "Notice has been" not in splitlines[index_line]:
                    # Build description line by line
                    description = f"{description} {splitlines[index_line]}"
                else:
                    # Stop looking for description lines
                    break
        description = clean_string(description)
        if description:
            return description
        raise Exception(
            f"Can't get docket entry description for court: {self.court_id}"
        )

    def _contains_attachments(self) -> bool:
        """Determines if the html notification contains attached documents.

        :returns: True if it contains otherwise False.
        """

        document_nodes = self.tree.xpath(
            '//strong[contains(., "Document description:")]'
        )
        if len(document_nodes) <= 1:
            return False
        return True

    def _contains_attachments_plain(self) -> bool:
        """Determines if the plain/txt notification contains attached documents.

        :returns: True if it contains otherwise False.
        """

        mail_body = self.tree.text_content()
        regex = r"^.*?The following document\(s\) are associated with this transaction:(.*?)$"

        find_attachments = re.findall(regex, mail_body, re.DOTALL)
        associated_documents = 0
        if find_attachments:
            splitlines = find_attachments[0].splitlines()
            for index_line in range(len(splitlines)):
                if "Document description:" in splitlines[index_line]:
                    associated_documents = +1
        if associated_documents <= 1:
            return False
        return True

    def _get_dockets(self) -> DocketType:
        """Get all the dockets mentioned in the notification.

        Right now multiple docket notifications are only supported for text/html
        NEF notifications since we don't have examples for text/plain or NDAs
        that mention multiple dockets. When we get one we'll log an error
        and get the example.

        :return: DocketType Dict
        """
        dockets = []
        if self.content_type == "text/plain":
            docket_number = self._get_docket_number_plain()
            docket = {
                "case_name": self._get_case_name_plain(),
                "docket_number": docket_number,
                "date_filed": self._get_date_filed(),
                "docket_entries": self._get_docket_entries(),
            }
            dockets.append(docket)
            # Cache the docket number for its later use.
            self.docket_numbers.append(docket_number)
        else:
            dockets_table = self.tree.xpath(
                "//table[contains(., 'Case Name:')]"
            )
            if self.appellate and len(dockets_table) > 1:
                raise NotImplementedError(
                    f"Received a potential multi-docket NDA notification. "
                    f"This is probably our chance to add support for it. "
                    f"court: {self.court_id}"
                )
            for docket_table in dockets_table:
                docket_number = self._get_docket_number(docket_table)
                # Cache the docket number and case name for its later use.
                self.docket_numbers.append(docket_number)
                docket = {
                    "case_name": self._get_case_name(docket_table),
                    "docket_number": docket_number,
                    "date_filed": self._get_date_filed(),
                    "docket_entries": self._get_docket_entries(docket_table),
                }
                dockets.append(docket)

            if len(dockets) > 1:
                # In multi-docket NEFs, the subject refers to only the short
                # description of the first item.
                for docket in dockets:
                    if docket["docket_entries"]:
                        docket["docket_entries"][0][
                            "short_description"
                        ] = self._get_short_description()
        return dockets

    def _get_docket_entries(
        self, current_node: HtmlElement = None
    ) -> List[DocketEntryType]:
        """Gets the full list of docket entries with document and sequence numbers

        :param  current_node: The relative lxml.HtmlElement
        :returns: List of docket entry dictionaries
        """

        case_url = None
        if self.content_type == "text/plain":
            description = self._get_description_plain()
            if description is not None:
                email_body = self.tree.text_content()
                regex = r"view the document:[\r\n]+([^\r\n]+)"
                url = re.findall(regex, email_body)
                if url:
                    document_url = url[0]
                else:
                    document_url = None
                document_number = self._get_document_number_plain()

                # Get Case URL for plain text version.
                regex = r"Case Number: .*? (https?:\/\/\S+)"
                match = re.search(regex, email_body)
                if match:
                    case_url = match.group(1)
        else:
            description = self._get_description(current_node)
            if description is not None:
                anchor = self._get_doc1_anchor(current_node)
                document_url = (
                    anchor.xpath("./@href")[0] if anchor is not None else None
                )
                if self._is_appellate():
                    document_number = None
                else:
                    document_number = self._get_document_number(current_node)

                # Get Case URL for HTML version.
                case_url = self._get_case_anchor(current_node)

        if description is not None:
            entries = [
                {
                    "date_filed": self._get_date_filed(),
                    "description": description,
                    "short_description": self._get_short_description(),
                    "document_url": document_url,
                    "document_number": document_number,
                    "pacer_doc_id": None,
                    "pacer_case_id": None,
                    "pacer_seq_no": None,
                    "pacer_magic_num": None,
                }
            ]
            if document_url is not None:
                entries[0]["pacer_doc_id"] = get_pacer_doc_id_from_doc1_url(
                    document_url
                )
                entries[0][
                    "pacer_magic_num"
                ] = get_pacer_magic_num_from_doc1_url(
                    document_url, self.appellate
                )
                if not self._is_appellate():
                    entries[0][
                        "pacer_case_id"
                    ] = get_pacer_case_id_from_doc1_url(document_url)
                    entries[0][
                        "pacer_seq_no"
                    ] = get_pacer_seq_no_from_doc1_url(document_url)

            # Fallback on the Case URL to get the pacer_case_id.
            if not entries[0]["pacer_case_id"] and case_url:
                entries[0]["pacer_case_id"] = self._get_case_id_from_case_url(
                    case_url
                )

            return entries
        return []

    def _parse_bankruptcy_short_description(self, subject: str) -> str:
        """Parse the short description of a bankruptcy case from the email
        subject. Subjects for bankruptcy varies a lot from court to court.
        This function supports parsing the short description for courts with
        known examples.

        :param subject: The email subject string.
        :return: The parsed short description.
        """

        if len(self.docket_numbers) > 1:
            # Since we don't have examples for bankruptcy multi docket NEF.
            # No short_description parsing support yet.
            return ""

        short_description = ""
        docket_number = self.docket_numbers[0]
        case_name = self.case_names[0]

        if self.court_id == "cacb" or self.court_id == "ctb":
            # In: 6:22-bk-13643-SY Request for courtesy Notice of Electronic Filing (NEF)
            # Out: Request for courtesy Notice of Electronic Filing (NEF)
            short_description = subject.split(docket_number)[-1]

            # Remove docket number traces "-AAA"
            regex = r"^-.*?\s"
            short_description = re.sub(regex, "", short_description)

        elif self.court_id == "njb":
            # In: Ch-11 19-27439-MBK Determination of Adjournment Request - Hollister Construc
            # Out: Determination of Adjournment Request
            short_description = subject.split(docket_number)[-1]
            short_description = short_description.rsplit("-", 1)[0]

            # Remove docket number traces "-AAA"
            regex = r"^-.*?\s"
            short_description = re.sub(regex, "", short_description)

        elif self.court_id == "nysb":
            # In: 22-22507-cgm Ch13 Affidavit Re: Gerasimos Stefanitsis
            # Out: Affidavit
            short_description = subject.split(case_name)[0]
            short_description = short_description.replace("Re:", "")
            short_description = short_description.split(docket_number)[-1]

            # Remove strings starting with "Ch" followed by a number
            regex = r"\bCh\d+\b"
            short_description = re.sub(regex, "", short_description)

            # Remove docket number traces "-AAA"
            regex = r"^-.*?\s"
            short_description = re.sub(regex, "", short_description)

        elif self.court_id == "pawb":
            # In: Ch-7 22-20823-GLT U LOCK INC Reply
            # Out: Reply
            short_description = subject.split(case_name)[-1]

        return short_description

    def _parse_appellate_short_description(self, subject: str) -> str:
        """Parse the short description of an appellate entry from the subject
        or from the footer notification, returns the better one.

        :param subject: The subject string from which to parse the short
        description.
        :return: The parsed short description
        """

        # Parse the short description from the notification footer.
        path = "//strong[contains(text(), 'Document Description: ')]/following-sibling::text()[1]"
        try:
            short_description_footer = self.tree.xpath(path)[0]
        except IndexError:
            short_description_footer = ""

        # Replace _ with whitespace in strings like Defective_Document_Notice
        short_description_footer = short_description_footer.replace("_", " ")

        # Sometimes the description in the footer only says "Main document."
        # Skip it.
        if short_description_footer == "Main document":
            short_description_footer = ""

        # Parse the description from the subject as a fallback.
        # In: 21-1975 New York State Telecommunicati v. James "Letter RECEIVED"
        # Out: Letter RECEIVED
        subject_split_case_name = subject.split(self.case_names[0])
        match = re.search(r'"(.*?)"', subject_split_case_name[-1])
        short_description_subject = ""
        if match:
            short_description_subject = match.group(1)

        # Select the longer short description, either from the subject or footer
        longer_short_description = max(
            short_description_subject, short_description_footer, key=len
        )
        return longer_short_description

    def _get_short_description(self) -> str:
        """Get the short description of a case from the subject string.

        :returns: The short description of the case.
        """

        if not self.subject:
            return ""
        subject = clean_string(self.subject)
        for case_name in self.case_names:
            # cases_names is a list of strings that can contain one or multiple
            # elements in multi-docket NEF where the case_name referenced in the
            # subject might change. This find the right case_name match.
            subject_split_case_name = subject.split(case_name)
            if len(subject_split_case_name) > 1:
                break

        if self.appellate:
            # Appellate notification.
            short_description = self._parse_appellate_short_description(
                subject
            )

        elif self.is_bankruptcy:
            # Bankruptcy notification.
            short_description = self._parse_bankruptcy_short_description(
                subject
            )
        else:
            # District notification.
            # In: Activity in Case 1:21-cv-01456-MN CBV, Inc. v. ChanBond, LLC Letter
            # Out: Letter
            short_description = subject_split_case_name[-1].strip()

        return clean_string(short_description)

    def _get_emaiL_recipients_without_links(self, recipient_lines):
        """Gets all the email recipients of the notification

        :returns: List of email recipients with names and email addresses
        """
        email_recipients = []
        for line in recipient_lines:
            if "@" in line:
                comma_separated = list(map(clean_string, line.split(",")))
                # The first element of comma_separated looks like "Stephen Breyer sbreyerguy52@hotmail.com"
                name_and_first_email = comma_separated[0].split(" ")
                # This re-joins so the name is by itself "Stephen Breyer"
                name = " ".join(name_and_first_email[:-1])
                # This is the leftover email in that first example "sbreyerguy52@hotmail.com"
                first_email = name_and_first_email[-1]
                # The remaining emails are the tail of the comma_separated list ["sbreyer@supremecourt.gov", "sbreyer@supremestreetwear.com"]
                other_emails = comma_separated[1:]
                email_recipients.append(
                    {
                        "name": name,
                        "email_addresses": [first_email] + other_emails,
                    }
                )
        return email_recipients

    def _get_email_recipients_with_links(self, text_content):
        """Gets all the email recipients of the notification if their emails are in links

        :returns: List of email recipients with names and email addresses
        """
        # Matching names in this format is a bit less reliable. May be worth coming back to.

        replacements = [
            (r"\n", ""),
            (
                r"\s{2,}|\t",
                " ",
            ),
            (
                r"\s,",
                "",
            ),
            (
                r"^.*mailed\sto:",
                "",
            ),
        ]
        for end_point in self.docket_numbers:
            replacements.append(
                (
                    f"{re.escape(end_point)}.*$",
                    "",
                )
            )

        for replacement in replacements:
            text_content = re.sub(replacement[0], replacement[1], text_content)

        recipient_parts = text_content.strip().split(" ")
        email_recipients = []
        for recipient_part in recipient_parts:
            if not len(email_recipients) and "@" not in recipient_part:
                email_recipients.append({"name": recipient_part})
            else:
                if not len(email_recipients):
                    email_recipients.append({"name": ""})
                last_recipient = email_recipients[-1]
                if "@" in recipient_part:
                    if not last_recipient.get("email_addresses"):
                        last_recipient["email_addresses"] = []
                    last_recipient["email_addresses"].append(
                        re.sub(r",", "", recipient_part)
                    )
                elif last_recipient.get("email_addresses") and len(
                    last_recipient["email_addresses"]
                ):
                    email_recipients.append({"name": recipient_part})
                else:
                    last_recipient["name"] += f" {recipient_part}"
        return list(
            filter(
                lambda recipient: recipient.get("email_addresses", False)
                and len(recipient.get("email_addresses")) > 0,
                email_recipients,
            )
        )

    def _get_email_recipients(self) -> List[Dict[str, Union[str, List[str]]]]:
        """Gets all the email recipients whether they come from plain text or more HTML formatting

        :returns: List of email recipients with names and email addresses
        """
        if self._is_appellate():
            path = '//strong[contains(., "Notice will be electronically mailed to")]/following-sibling::'
            recipient_lines = self.tree.xpath(f"{path}text()")
        else:
            path = '//b[contains(., "Notice has been electronically mailed to")]/following-sibling::'
            recipient_lines = self.tree.xpath(f"{path}text()")
            link_lines = self.tree.xpath(f"{path}a")
            if len(link_lines):
                return self._get_email_recipients_with_links(
                    self.tree.xpath(
                        'string(//b[contains(., "Notice has been electronically mailed to")]/parent::node())'
                    )
                )
        if not recipient_lines:
            path = '//b[contains(., "Notice will be electronically mailed to")]/following-sibling::'
            recipient_lines = self.tree.xpath(f"{path}text()")

        return self._get_email_recipients_with_links(" ".join(recipient_lines))

    def _get_email_recipients_plain(
        self,
    ) -> List[Dict[str, Union[str, List[str]]]]:
        """Gets all the email recipients whether they come from plain text or more HTML formatting

        :returns: List of email recipients with names and email addresses
        """
        email_recipients = []
        mail_body = self.tree.text_content()
        regex = r"^.*?Notice has been electronically mailed to:(.*?)$"

        # Return all lines after recipients begins
        find_emails = re.findall(regex, mail_body, re.DOTALL)
        if find_emails:
            email_lines = find_emails[0]
            splitlines = email_lines.splitlines()
            for index_line in range(len(splitlines)):
                if "@" in splitlines[index_line]:
                    email_separated = list(
                        map(clean_string, splitlines[index_line].split(","))
                    )
                    # Obtains comma separated email addresses ["sbreyer@supremecourt.gov", "sbreyer@supremestreetwear.com"]
                    name = clean_string(splitlines[index_line - 1])
                    # Obtains recipient name
                    email_recipients.append(
                        {
                            "email_addresses": email_separated,
                            "name": name,
                        }
                    )
                if "Notice will be delivered" in splitlines[index_line]:
                    # Stop looking for email addresses
                    break
        return email_recipients


class S3NotificationEmail(NotificationEmail):
    """A subclass of the NotificationEmail report. This handles all the S3 specific format issues that come from
    SES emails automatically archived in S3.
    """

    def _combine_lines_with_proper_spaces(self, text):
        """Re-composes S3 line breaks to have proper spacing depending on line ending character

        :returns: String with spacing as read normally
        """
        lines = text.split("\n")
        combined = ""
        last_line_match = False
        for line in lines:
            match = re.search(r"=$", line)
            if match:
                combined += re.sub(r"=$", "", line)
                last_line_match = True
            elif last_line_match:
                combined += line
                last_line_match = False
            else:
                combined += f" {line}"
        return combined

    def _html_from_s3_email(self, text):
        """Pulls the HTML content, parsed with line breaks normalized for from the S3 email file

        :returns: String with proper replacements for normal HTML parsing
        """
        # Remove line ends form S3 content
        cleaned_s3_line_ends = self._combine_lines_with_proper_spaces(text)
        return cleaned_s3_line_ends

    def _parse_text(self, text):
        """MIME Parser from file text, text/html and text/plain messages are supported.
        This obtains the email payload decoded as UTF-8.
        """
        message = email.message_from_string(text)
        self.subject = message.get("Subject")
        if message.is_multipart():
            # Checks if the email contains an attached image.
            if any(
                part.get_content_maintype() == "image"
                for part in message.walk()
            ):
                self.image_attached = True

            for part in message.walk():
                c_type = part.get_content_type()
                # If multipart message, parse text/html message
                if c_type == "text/html":
                    body = part.get_payload(decode=True)  # decode
                    self.content_type = "text/html"
                    break

                elif c_type == "text/plain":
                    body = part.get_payload(decode=True)
                    self.content_type = "text/plain"
                    break
        else:
            # If not multipart, parse either text/html or text/plain message
            for part in message.walk():
                c_type = part.get_content_type()
                c_dispo = str(part.get("Content-Disposition"))

                if c_type == "text/html":
                    body = part.get_payload(decode=True)
                    self.content_type = "text/html"
                    break

                elif c_type == "text/plain" and "attachment" not in c_dispo:
                    body = message.get_payload(decode=True)
                    self.content_type = "text/plain"
                    break

        try:
            # Try to decode email body using utf-8
            email_body = body.decode("utf-8")
        except UnicodeDecodeError:
            # If it fails fallback on iso-8859-1
            email_body = body.decode("iso-8859-1")
        if self.content_type == "text/plain":
            return super()._parse_text(email_body)
        elif self.content_type == "text/html":
            html_only = self._html_from_s3_email(email_body)
            return super()._parse_text(html_only)



# Author: Michael Lissner
# History:
# - 2013-06-11: Birth.
# - 2013-08-06: Revised by Brian Carver
# - 2014-08-05: Updated URL by mlr
# - 2021-12-28: Updated by flooie

import re
from typing import Any, Dict

from juriscraper.lib.html_utils import get_row_column_text
from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Published"
        self.url = "https://ujs.sd.gov/Supreme_Court/opinions.aspx"

    def _process_html(self):
        rows = self.html.xpath(
            "//div[@id='ContentPlaceHolder1_ChildContent1_UpdatePanel_Opinions']//tbody/tr"
        )
        for row in rows:
            cite = re.findall(r"\d{4} S\.D\. \d+", get_row_column_text(row, 2))
            if not cite:
                continue

            # https://ujs.sd.gov/uploads/sc/opinions/2928369ef9a6.pdf
            # We abstract out the first part of the docket number here
            # And process the full docket number in the `extract_from_text` method
            # Called after the file has been downloaded.
            docket = row.xpath(".//td[2]/a/@href")[0].split("/")[-1][:5]
            self.cases.append(
                {
                    "date": get_row_column_text(row, 1),
                    "name": titlecase(
                        get_row_column_text(row, 2).rsplit(",", 1)[0]
                    ),
                    "citation": cite[0],
                    "url": row.xpath(".//td[2]/a/@href")[0],
                    "docket": docket,
                }
            )

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Can we extract the date filed from the text?

        :param scraped_text: The content of the document downloaded
        :return: Metadata to be added to the case
        """

        # The docket number appears to be the first text on the page.
        # So I crop the text to avoid any confusion that might occur in the
        # body of an opinion.
        docket = re.findall(r"#\d+.*-.-\w{3}", scraped_text)[0][:20]
        metadata = {
            "Docket": {
                "docket_number": docket,
            },
        }
        return metadata



from lxml import html

from juriscraper.AbstractSite import logger
from juriscraper.lib.exceptions import InsanityException
from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = (
            "http://www.ca10.uscourts.gov/opinions/new/daily_decisions.rss"
        )
        self.court_id = self.__module__

    def _get_case_names(self):
        """Case name parsing
        Expected value for title_string:
            &lt;p&gt;Bazan-Martinez v. Garland&lt;/p&gt;
        """
        case_names = []
        for title_string in self.html.xpath("//item/title/text()"):
            try:
                p_element = html.etree.fromstring(str(title_string))
                title_string = p_element.xpath("/p/text()")[0]
                case_names.append(title_string)
            except:
                logger.error(f"Error while parsing case name: {title_string}")
                raise InsanityException(
                    f"Error while parsing case name: {title_string}"
                )
        return case_names

    def _get_download_urls(self):
        return [
            html.tostring(e, method="text").decode()
            for e in self.html.xpath("//item/link")
        ]

    def _get_case_dates(self):
        """Case date parsing
        Expected value for date_string:
            &lt;span class=&quot;date-display-single&quot; property=&quot;dc:date&quot; datatype=&quot;xsd:dateTime&quot; content=&quot;2021-11-16T00:00:00-07:00&quot;&gt;Tue Nov 16 2021&lt;/span&gt;
        """
        dates = []
        for date_string in self.html.xpath("//item/pubdate/text()"):
            try:
                span_element = html.etree.fromstring(str(date_string))
                date_string = span_element.xpath("/span/text()")[0]
                dates.append(convert_date_string(date_string))
            except:
                logger.error(f"Error while parsing case date: {date_string}")
                raise InsanityException(
                    f"Error while parsing case date: {date_string}"
                )
        return dates

    def _get_docket_numbers(self):
        """Case docket parsing
        Expected content in description tag:
            Docket#: 21-6001 - Date Issued: Mon Nov 15 2021 - Unpublished Order and Judgment
        """
        return [
            e.split(" - ")[0].split(":")[1]
            for e in self.html.xpath("//item/description/text()")
        ]

    def _get_precedential_statuses(self):
        """Case precedential status parsing
        Expected content in description tag:
            Docket#: 21-5062 - Date Issued: Fri Nov 12 2021 - Unpublished Order and Judgment
        Status:
            - Published: "Published Opinion"
            - Unpublished: "Unpublished Order and Judgment"
        """
        return [
            "Published"
            if "published opinion" in e.split(" - ")[2].lower()
            else "Unpublished"
            for e in self.html.xpath("//item/description/text()")
        ]

    def _get_lower_courts(self):
        """Case lower court name parsing
        namescpace "dc": "http://purl.org/dc/elements/1.1/"
        Tags:
            - <dc:creator>Board of Immigration Appeals</dc:creator>
        """
        return [
            e
            for e in self.html.xpath(
                "//item/creator/text()",
                namespaces={"dc": "http://purl.org/dc/elements/1.1/"},
            )
        ]



"""Scraper for Connecticut Supreme Court
CourtID: conn
Court Short Name: Conn.
Author: Asadullah Baig <asadullahbeg@outlook.com>
History:
    - 2014-07-11: created
    - 2014-08-08, mlr: updated to fix InsanityError on case_dates
    - 2014-09-18, mlr: updated XPath to fix InsanityError on docket_numbers
    - 2015-06-17, mlr: made it more lenient about date formatting
    - 2016-07-21, arderyp: fixed to handle altered site format
    - 2017-01-10, arderyp: restructured to handle new format use case that includes opinions without dates and flagged for 'future' publication
    - 2022-02-02, satsuki-chan: Fixed docket and name separator, changed super class to OpinionSiteLinear
"""

import re
from datetime import date

from juriscraper.lib.string_utils import normalize_dashes
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.year = date.today().strftime("%y")
        self.url = f"http://www.jud.ct.gov/external/supapp/archiveAROsup{self.year}.htm"
        self.status = "Published"
        self.cases = []
        self.docket_regex = r"SC\d+"

    def _process_html(self) -> None:
        """Process the html and extract out the opinions

        :return: None
        """
        date = None
        for row in self.html.xpath(
            r"//*[re:match(text(), '.*\d{1,2}/\d{1,2}/\d{2,4}')] | //table[@id='AutoNumber1']//li",
            namespaces={"re": "http://exslt.org/regular-expressions"},
        ):
            dates = re.findall(r"\d{1,2}/\d{1,2}/\d{2,4}", row.text_content())
            if dates:
                date = dates[0]
                continue
            if not date:
                continue
            link_node = row.xpath(".//a")[0]
            link_text = link_node.text_content().strip()
            name = normalize_dashes(row.text_content().replace(link_text, ""))
            name = re.sub(r" - ", "", name).strip()
            dockets = ", ".join(re.findall(self.docket_regex, link_text))
            self.cases.append(
                {
                    "date": date,
                    "url": link_node.attrib["href"],
                    "docket": dockets,
                    "name": name,
                }
            )



import re

from juriscraper.opinions.united_states.federal_appellate import cadc


class Site(cadc.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://www.cadc.uscourts.gov/internet/judgments.nsf/uscadcjudgments.xml"
        self.court_id = self.__module__

    def _get_case_names(self):
        return [
            e.split(", ", 1)[1]
            for e in self.html.xpath("//item/description/text()")
        ]

    def _get_docket_numbers(self):
        return [
            re.split("Judgment in Case |,", e)[1]
            for e in self.html.xpath("//item/title/text()")
        ]

    def _get_precedential_statuses(self):
        return ["Unpublished" for _ in range(0, len(self.case_names))]



"""Scraper for Supreme Court of Virgin Islands
CourtID: vi
Court Short Name: Virgin Islands
Author: William Edward Palin
History:
  2023-01-21: Created by William Palin
"""
import urllib.parse
from datetime import date, datetime, timedelta

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            f"https://supreme.vicourts.org/court_opinions/published_opinions"
        )
        self.status = "Published"
        today = date.today()
        self.last_month = today - timedelta(days=30)

    def _process_html(self):
        if self.test_mode_enabled():
            self.last_month = datetime(2022, 12, 25).date()
        for s in self.html.xpath(".//tr/td/.."):
            cells = s.xpath(".//td")
            judges = cells[3].text_content()
            if not cells[0].text_content():
                continue
            dt = convert_date_string(cells[1].text_content())
            if dt < self.last_month:
                continue
            u = s.xpath(".//td/a/@href")[0]
            url = urllib.parse.quote(u, safe="/:")
            self.cases.append(
                {
                    "name": cells[0].text_content(),
                    "date": cells[1].text_content(),
                    "docket": cells[2].text_content(),
                    "judges": judges,
                    "url": url,
                }
            )



# Author: Michael Lissner
# Date created: 2013-05-23

from datetime import date

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        year = date.today().year
        self.url = (
            "http://www.courts.state.hi.us/opinions_and_orders/opinions?yr=%s"
            % year
        )
        self.back_scrape_iterable = list(range(2010, year + 1))
        self.court_id = self.__module__
        self.target_id = "S.Ct"
        self.cases = []

    def _download(self, request_dict={}):
        html = super()._download(request_dict)
        self.extract_cases_from_table(html)
        return html

    def extract_cases_from_table(self, html):
        path_rows = '//div[@id="table-content"]/table/tbody/tr'
        path_target = "./td[2]/div/span/text()"
        for row in html.xpath(path_rows):
            court = row.xpath(path_target)
            if court and court[0].strip() == self.target_id:
                case_text = row.xpath(
                    "./td[4]/p/text() | ./td[4]/p/span/text()"
                )[0]
                court_text = row.xpath("./td[5]/text()")
                self.cases.append(
                    {
                        "url": row.xpath("./td[3]/a/@href")[0],
                        "date": row.xpath("./td[1]/text()")[0],
                        "name": case_text.split(" (")[0],
                        "docket": row.xpath("./td[3]/a/text()")[0],
                        "court": court_text[0] if court_text else False,
                    }
                )

    def _get_download_urls(self):
        return [case["url"] for case in self.cases]

    def _get_case_names(self):
        return [case["name"] for case in self.cases]

    def _get_case_dates(self):
        return [convert_date_string(case["date"]) for case in self.cases]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.cases)

    def _get_docket_numbers(self):
        return [case["docket"] for case in self.cases]

    def _get_lower_courts(self):
        return [case["court"] for case in self.cases]

    def _download_backwards(self, year):
        if int(year) < 2010:
            raise Exception("the haw portal does not include pre-2010 cases")

        self.url = (
            "http://www.courts.state.hi.us/opinions_and_orders/opinions?yr=%s"
            % year
        )
        self.html = self._download()

        # Loop over all pages
        pages = self.html.xpath('//a[@class="page-numbers"]/text()')
        if pages:
            last_page = int(pages[-1])
            for page in range(2, last_page + 1):
                self.url = (
                    "http://www.courts.state.hi.us/opinions_and_orders/opinions/page/%d?yr=%s"
                    % (page, year)
                )
                self.html = self._download()



# Scraper and Back Scraper for New York Appellate Term 2nd Dept.
# CourtID: nyappterm_2nd
# Court Short Name: NY
# Author: Andrei Chelaru
# Reviewer:
# Date: 2015-10-30

from juriscraper.opinions.united_states.state import nyappterm_1st


class Site(nyappterm_1st.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court = "Appellate Term, 2d Dept"
        self.parameters.update({"court": self.court})



"""Scraper for U.S. Merit Systems Protection Board
CourtID: mspb
Court Short Name: MSPB
Author: Jon Andersen
Reviewer: mlr
Date created: 1 Sep 2014
Type: Precedential
"""
import json

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Published"
        self.url = "https://www.mspb.gov/decisions/precedential/PrecedentialDecisions_Manifest_Table.json"
        self.base = "https://www.mspb.gov/decisions/precedential"

    def _process_html(self):
        if self.test_mode_enabled():
            with open(self.url) as file:
                self.html = json.load(file)
        for row in self.html["data"]:
            url = row["FILE_NAME"]
            name = f"{row['APL_FIRST_NAME']} {row['APL_LAST_NAME']} v. {row['AGENCY']}"
            self.cases.append(
                {
                    "citation": row["DECISION_NUMBER"],
                    "url": f"{self.base}/{url}",
                    "docket": row["DOCKET_NBR"],
                    "name": name,
                    "date": row["ISSUED_DATE"].replace("/", "-"),
                }
            )



"""Scraper for the Navy-Marine Corps Court of Criminal Appeals
CourtID: nmcca
Court Short Name:
Reviewer: mlr
History:
    15 Sep 2014: Created by Jon Andersen
"""

from juriscraper.lib.string_utils import convert_date_string, titlecase
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.jag.navy.mil/courts/opinion_archive.htm"
        self.court_id = self.__module__
        self.back_scrape_iterable = list(range(2013, 2004 - 1, -1))

    def _get_case_dates(self):
        # this court makes a lot of typos apparently
        typos = {
            "6/13/30/13": "6/13/13",
            "6/11/30/13": "6/11/13",
            "02/12/09 & 12/04/08": "02/12/09",
            "006/08/2020": "06/08/2020",
        }

        dates = []
        path = "//table/tbody/tr/td[3]//text()"
        for ds in self.html.xpath(path):
            ds = typos[ds] if ds in typos else ds
            dates.append(convert_date_string(ds))
        return dates

    def _get_case_names(self):
        path = "//table/tbody/tr/td[1]/text()"
        return [titlecase(text) for text in self.html.xpath(path)]

    def _get_download_urls(self):
        path = "//table/tbody/tr/td[4]/a[1]/@href"
        return [e for e in self.html.xpath(path)]

    def _get_docket_numbers(self):
        path = "//table/tbody/tr/td[2]//text()"
        return [docket_number for docket_number in self.html.xpath(path)]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_dates)

    def _download_backwards(self, year):
        self.url = (
            "http://www.jag.navy.mil/courts/opinion_archive_%d.htm" % year
        )
        self.html = self._download()



"""Scraper for Colorado Supreme Court
CourtID: colo
Court Short Name: Colo.
Author: Philip Ardery
Reviewer: mlr
Date created: 2016-06-03
Contact: Email "Internet and Technology" staff listed at http://www.cobar.org/staff
         they usually fix issues without responding to the emails directly. You can
         also try submitting the form here: http://www.cobar.org/contact
History:
    - 2022-01-31: Updated by William E. Palin
    - 2023-01-05: Updated by William E. Palin
"""
import datetime
import re

from lxml.html import fromstring, tostring

from juriscraper.OpinionSiteLinearWebDriven import OpinionSiteLinearWebDriven


class Site(OpinionSiteLinearWebDriven):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.year = datetime.date.today().year
        self.month = datetime.date.today().month
        self.url = f"https://www.courts.state.co.us/Courts/Supreme_Court/Case_Announcements/Index.cfm?year={self.year}&month={self.month}&Submit=Go"
        self.uses_selenium = True

    def _process_html(self):
        if not self.test_mode_enabled():
            self.initiate_webdriven_session()

        if not self.test_mode_enabled():
            links = self.html.xpath(".//a[@class='touchable-link']")
        else:
            links = ["fake_link"]

        for link in links:
            if self.test_mode_enabled():
                super()._download()
                date_filed = "December 19, 2022"
            else:
                date_filed = link.text_content()
                self.webdriver.get(link.get("href"))
                # Wait for pdf to render
                self.webdriver.implicitly_wait(10000)
                # And click next twice to make sure we fully render the PDF content
                self.find_element_by_id("next").click()
                self.find_element_by_id("next").click()

                self.html = fromstring(self.webdriver.page_source)

            for pg in self.html.xpath(".//div[@class='page']"):
                if not pg.xpath(".//span[@role='link']/span/@aria-owns"):
                    continue
                anno_id = pg.xpath(".//span[@role='link']/span/@aria-owns")[0]
                pg_content = []
                for element in pg.xpath(".//span"):
                    if element.get("class") == "markedContent":
                        continue
                    pg_content.append(element.text_content())
                pg_text = " ".join(pg_content)

                case_name = self.get_case_name(pg)
                citation = re.findall(r"20\d{2} CO \d{1,3}M?", pg_text)
                if not citation:
                    # This is how we identify a page with an opinion
                    continue
                citation = citation[0]
                judges = ""
                m = re.findall(r"JUSTICE\s+\b([\wÁ]+)\b", pg.text_content())
                if m:
                    judges = ", ".join(m).replace("Á", "A")

                urls = pg.xpath(
                    f"//a[@id='pdfjs_internal_id_{anno_id}']/@href"
                )
                if not urls:
                    # One time an opinion wasn't linked.
                    continue
                url = urls[0]
                if "Opinions" not in url:
                    continue
                docket = url.split("/")[-1][:-4]

                self.cases.append(
                    {
                        "date": date_filed,
                        "name": case_name,
                        "docket": docket,
                        "status": "Published",
                        "url": url,
                        "citation": citation,
                        "judges": judges,
                    }
                )
        if not self.test_mode_enabled():
            self.webdriver.quit()

    def get_case_name(self, pg) -> str:
        """Get case name from page content

        :param pg: Page Element
        :return: Cleaned case name
        """
        start = False
        content = []
        for element in pg.xpath(".//*"):
            if (
                b"markedContent" in tostring(element)
                or element.text_content() == ""
            ):
                continue
            # This denotes a bolded field e.g. Respondent etc.
            if ":" in element.text_content():
                start = True
                continue
            # IDs an italics, e.g. en banc at the end of the name
            if b"transform: scaleX(0.8" in tostring(element):
                start = False
            if not start:
                continue

            content.append(element.text_content())
        case_name = " ".join(content[:-1]).replace(" ,", "").replace(" .", "")
        return re.sub(r"\s{2,}", " ", case_name.strip())



import difflib
import re
import string

from .string_utils import harmonize

STOP_WORDS = (
    r"a|an|and|as|at|but|by|en|etc|for|if|in|is|of|on|or|the|to|v\.?|via|"
    r"vs\.?|et|al|appellants?|defendants?|administrator|"
    "plaintiffs?|error|others|against|ex|parte|complainants?|original|"
    "claimants?|devisee|executrix|executor|comm(issione)?r"
)
STOP_WORDS_RE = re.compile(r"^(%s)$" % STOP_WORDS)


def normalize_phrase(phrase):
    """Clean up words or phrases before sending them to be compared.

    - Harmonize things like United States, USA, etc.
    - Normalize to lower case.
    - Remove punctuation.
    - Remove stop words.
    """
    phrase = harmonize(phrase)
    phrase = phrase.lower()

    # strip punctuation
    exclude = set(string.punctuation)
    phrase = "".join(ch for ch in phrase if ch not in exclude)

    words = re.split("[\t ]", phrase)
    result = []
    for word in words:
        word = STOP_WORDS_RE.sub("", word)
        result.append(word)
    return "".join(result)


def get_closest_match_index(word, possibilities):
    """Find the string that is most similar to the target string. Uses difflib's
    SequenceMatcher under the covers.

    :param word: The string to match to.
    :param possibilities: A list of possible matches.
    :return: The index of the closest matching possibility, if any. Else, return
    None.
    """
    word = normalize_phrase(word)
    possibilities = [normalize_phrase(x) for x in possibilities]
    try:
        match = difflib.get_close_matches(
            word, possibilities, n=1, cutoff=0.1
        )[0]
    except IndexError:
        # No good matches.
        return None
    return possibilities.index(match)



# Author: Phil Ardery
# Date created: 2017-01-27
# Contact: 501-682-9400 (Administrative Office of the Curt)

import datetime
import re
from typing import Any, Dict

import feedparser

from juriscraper.lib.string_utils import normalize_dashes, titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    """This scraper has an intential 10 opinion limit to avoid lexum cap."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court = "supremecourt"
        self.url = f"https://opinions.arcourts.gov/ark/{self.court}/en/rss.do"
        self.year = datetime.date.today().year
        self.cite_regex = r"\d{2,4} Ark\. \d+"

    def _process_html(self) -> None:
        if self.test_mode_enabled():
            self.year = 2022
        feed = feedparser.parse(self.request["response"].content)
        for item in feed["entries"]:
            if not re.findall(self.cite_regex, item["title"], re.I | re.M):
                continue
            name, cite, date = item["title"].split("\n")
            name = titlecase(name.strip(" -"))
            cite = cite.strip(" -")
            date = date.strip(" -")

            if int(date[-4:]) != self.year:
                # The RSS feed has the most recent opinions and then dozens from the 1960's.  Not sure why
                # but we dont want to collect those.
                continue
            if len(self.cases) >= 10:
                # Limit to max of 10 cases per crawl to stay under limit.
                # We assume it is the same for NM (lexum) at 50/min 100/5min
                continue
            doc_id = re.findall(r"\d{4,}", item["link"])[0]
            url = f"https://opinions.arcourts.gov/ark/{self.court}/en/{doc_id}/1/document.do"
            self.cases.append(
                {
                    "date": date,
                    "docket": "",
                    "name": name,
                    "citation": cite,
                    "url": url,
                    "status": "Published",
                }
            )

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Pass scraped text into function and return data as a dictionary

        :param scraped_text: Text of scraped content
        :return: metadata
        """
        normalized_content = normalize_dashes(scraped_text)
        match = re.findall(r"No\. (\w+-\d+-\d+)", normalized_content)
        docket_number = match[0] if match else ""
        metadata = {
            "OpinionCluster": {
                "docket_number": docket_number,
            },
        }
        return metadata



"""
Scraper for Utah Court of Appeals
CourtID: utahctapp
Court Short Name: UT Ct. App.
History:
 2021-12-16  Created by William Palin
"""
from datetime import date

from juriscraper.opinions.united_states.state import utahctapp


class Site(utahctapp.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.back_scrape_iterable = list(range(2012, date.today().year))

    def _download_backwards(self, year: str) -> None:
        """Download the page for each year in the backscrape_iterable.

        :param year: The year to download
        :return: None
        """
        ending = "htm" if int(year) <= 2014 else "asp"
        self.url = (
            f"https://www.utcourts.gov/opinions/appopin/index-{year}.{ending}"
        )
        self.html = self._download()
        self._process_html()



import datetime
import re
from typing import Any, Dict

from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    """This site has an artificial limit of 50/1 minute and 100/5 minutes.

    To stay under that cap we are going to just request the first 10 opinions.  Judging on the number of opinions
    filed in each court we should be fine.

    Additionally, we moved docket number capture to PDF extraction, to limit the number of requests.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.year = datetime.date.today().year
        self.url = f"https://nmonesource.com/nmos/nmsc/en/{self.year}/nav_date.do?iframe=true"

    def _process_html(self):
        rows = self.html.xpath(
            ".//li[contains(./@class, 'list-item-expanded')]"
        )
        for row in rows:
            url = row.xpath(
                ".//a[contains(@title, 'Download the PDF version')]/@href"
            )[0]
            name = (
                row.xpath(".//span[@class='title']")[0].text_content().strip()
            )
            date = (
                row.xpath(".//span[@class='publicationDate']")[0]
                .text_content()
                .strip()
            )
            cite = row.xpath(".//span[@class='citation']")
            metadata = row.xpath(".//div[@class='subMetadata']/span/text()")
            if cite:
                citation = cite[0].text_content().strip()
            else:
                citation = ""
            if metadata:
                status = (
                    "Published"
                    if "Reported" in metadata[-1]
                    else "Unpublished"
                )
            else:
                status = "Unpublished"

            if len(self.cases) >= 10:
                # Limit to max of 10 cases per crawl to stay under limit.
                continue

            self.cases.append(
                {
                    "date": date,
                    "docket": "",
                    "name": titlecase(name),
                    "citation": citation,
                    "url": url,
                    "status": status,
                }
            )

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Pass scraped text into function and return data as a dictionary

        :param scraped_text: Text of scraped content
        :return: metadata
        """
        docket_number = re.findall(r"N[oO]\.\s(.*)", scraped_text)[0]
        metadata = {
            "OpinionCluster": {
                "docket_number": docket_number,
            },
        }
        return metadata



"""
Author: William Palin
Date created: 2023-01-16
Scraper for the Decisiones del Tribunal Supremo
CourtID: pr
Court Short Name: Puerto Rico
"""

from datetime import date

from dateparser import parse

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        year = date.today().year
        self.url = f"https://poderjudicial.pr/index.php/tribunal-supremo/decisiones-del-tribunal-supremo/decisiones-del-tribunal-supremo-{year}/"
        self.status = "Published"

    def _process_html(self):
        for row in self.html.xpath(".//table/tbody/tr/td/a/@href/../../.."):
            cells = row.xpath(".//td")
            citation = cells[0].text_content()
            url = cells[0].xpath(".//a/@href")[0]
            date_obj = parse(cells[4].text_content(), languages=["es"])
            if not date_obj:
                # This is to handle the junk on the website in one row
                continue
            self.cases.append(
                {
                    "name": cells[3].text_content(),
                    "url": url,
                    "citation": citation,
                    "docket": cells[2].text_content(),
                    "date": str(date_obj.date()),
                }
            )



"""Scraper for Supreme Court of Maine
CourtID: me
Court Short Name: Me.
Author: Brian W. Carver
Date created: June 20, 2014

History:
  2014-06-25 (est): Added code for additional date formats.
  2014-07-02: Was receiving InsanityException and tweaked date code to get some
              missing dates.
  2014-12-15: Fixes insanity exception by tweaking the XPaths.

  2022-01-06: This scraper is not maintained. Future work to gather this
              data should be done by scraping the CourtListener API
              https://www.courtlistener.com/api/rest/v3/clusters/?docket__court__id=me
"""

from lxml import html

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.courts.maine.gov/courts/sjc/opinions.html"
        self.path_root = '//table[contains(.//th[1], "Opinion")]'

    def _get_cell_path(self, cell_number: int, subpath: str = "") -> str:
        path = '//table[contains(.//th[1], "Opinion")]//td[%d]'
        return f"{path}/{subpath}" if subpath else path

    def _get_download_urls(self):
        path = f"{self.path_root}//td[2]/a[1]/@href"
        return list(self.html.xpath(path))

    def _get_case_names(self):
        case_names = []
        path = f"{self.path_root}//td[2]/a[1]"
        for e in self.html.xpath(path):
            s = html.tostring(e, method="text", encoding="unicode")
            case_names.append(s)
        return case_names

    def _get_case_dates(self):
        dates = []
        path = f"{self.path_root}//td[3]"
        for cell in self.html.xpath(path):
            date_string = cell.text_content().replace("Aguust", "August")
            dates.append(convert_date_string(date_string))
        return dates

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_citations(self):
        path = f"{self.path_root}//td[1]//text()"
        return list(self.html.xpath(path))



import time
from datetime import date

from lxml import html

from juriscraper.opinions.united_states.federal_special import cit


class Site(cit.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.cit.uscourts.gov/SlipOpinions/SlipOps-2012.html"
        self.court_id = self.__module__

    def _get_download_urls(self):
        return [t for t in self.html.xpath("//table[4]//tr/td[1]//a/@href")]

    def _get_citations(self):
        return [t for t in self.html.xpath("//table[4]//tr/td[1]//a/text()")]

    def _get_case_names(self):
        # Exclude confidential rows by ensuring there is a sibling row that
        # contains an anchor (which confidential cases do not)
        # There is also one stray case within a <p> tag we have to catch.
        return [
            s
            for s in self.html.xpath(
                "//table[4]//tr[position() > 1]/td[2][../td//a]/text()[1] | //table[4]//tr[position() > 1]/td[2][../td//a]/p/text()[1]"
            )
        ]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_case_dates(self):
        # This does not capture the release dates for the errata documents.
        # The errata release date is listed in column 2. This will use the
        # original release date instead.
        dates = []
        date_formats = ["%m/%d/%Y", "%m/%d/%y"]
        for date_string in self.html.xpath(
            "//table[4]//tr/td[3][../td//a]/text()"
        ):
            for date_format in date_formats:
                try:
                    d = date.fromtimestamp(
                        time.mktime(time.strptime(date_string, date_format))
                    )
                    dates.append(d)
                    break
                except ValueError:
                    # Try the next format
                    continue
        return dates

    def _get_docket_numbers(self):
        docket_numbers = []
        for e in self.html.xpath(
            "//table[4]//tr[position() > 1]/td[4][../td//a]"
        ):
            docket_numbers.append(
                html.tostring(e, method="text", encoding="unicode").strip()
            )
        return docket_numbers

    def _get_judges(self):
        judges = []
        for e in self.html.xpath(
            "//table[4]//tr[position() > 1]/td[5][../td//a]"
        ):
            s = html.tostring(e, method="text", encoding="unicode")
            judges.append(s)
        return judges

    def _get_nature_of_suit(self):
        return [
            t for t in self.html.xpath("//table[4]//tr/td[6][../td//a]/text()")
        ]



import time
from datetime import date

import certifi
import requests
from lxml import html

from juriscraper.AbstractSite import logger
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.base_url = (
            "http://digitalcommons.law.villanova.edu/thirdcircuit_{year}/"
        )
        self.url = self.base_url.format(year=date.today().year)
        self.court_id = self.__module__
        self.back_scrape_iterable = list(range(2003, date.today().year + 1))
        self.case_xpath = "//p[@class='article-listing']/a/@href"
        self.next_page_xpath = "//div[@class='adjacent-pagination']//li[@class='active']//following-sibling::li/a/@href"

    def _download(self, request_dict={}):
        if self.test_mode_enabled():
            # Note that this is returning a list of HTML trees.
            html_trees = [
                super()._download(request_dict=request_dict),
            ]
        else:
            html_l = super()._download(request_dict)
            html_trees = self._get_case_html_page([], html_l, request_dict)

            # handle the next page data
            if html_l.xpath(self.next_page_xpath):
                self.url = html_l.xpath(self.next_page_xpath)[0]
                html_next_trees = self._download(request_dict)
                html_trees.extend(html_next_trees)
        return html_trees

    def _get_case_html_page(self, html_trees, html_l, request_dict):
        """Gets each of the individual case pages"""
        s = requests.session()
        for case_url in html_l.xpath(self.case_xpath):
            logger.info(f"  Getting sub-page at: {case_url}")
            r = s.get(
                case_url,
                headers={"User-Agent": "Juriscraper"},
                verify=certifi.where(),
                timeout=60,
                **request_dict,
            )

            r.raise_for_status()

            # If the encoding is iso-8859-1, switch it to cp1252 (a superset)
            if r.encoding == "ISO-8859-1":
                r.encoding = "cp1252"

            # Grab the content
            text = self._clean_text(r.text)
            html_tree = html.fromstring(text)
            html_tree.make_links_absolute(self.url)

            remove_anchors = lambda url: url.split("#")[0]
            html_tree.rewrite_links(remove_anchors)
            html_trees.append(html_tree)
        return html_trees

    def _get_case_names(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(html_tree.xpath("//div[@id='title']//a/text()")[0])
            except IndexError:
                data.append("")
        return data

    def _get_download_urls(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(html_tree.xpath("//div[@id='title']//a/@href")[0])
            except IndexError:
                data.append("")
        return data

    def _get_case_dates(self):
        data = []
        for html_tree in self.html:
            try:
                date__ = html_tree.xpath(
                    "//div[@id='publication_date']/p/text()"
                )[0]
                data.append(
                    date.fromtimestamp(
                        time.mktime(time.strptime(date__, "%m-%d-%Y"))
                    )
                )
            except IndexError:
                data.append("")
        return data

    def _get_docket_numbers(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(
                    html_tree.xpath("//div[@id='comments']//p/text()")[0]
                    .replace("No.", "")
                    .strip()
                )
            except IndexError:
                data.append("")
        return data

    def _get_precedential_statuses(self):
        data = []
        for html_tree in self.html:
            try:
                status = html_tree.xpath("//div[@id='precedent']//p/text()")[0]
                if "non" in status.lower():
                    data.append("Unpublished")
                else:
                    data.append("Published")
            except IndexError:
                data.append("Unknown")
        return data

    def _get_lower_courts(self):
        data = []
        for html_tree in self.html:
            try:
                data.append(
                    html_tree.xpath("//div[@id='abstract']//p/text()")[0]
                )
            except IndexError:
                data.append("")
        return data

    def _download_backwards(self, d):
        self.url = self.base_url.format(year=d)
        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200

    def _post_parse(self):
        """This will remove the cases without a case name"""
        to_be_removed = [
            index
            for index, case_name in enumerate(self.case_names)
            if not case_name
        ]

        for attr in self._all_attrs:
            item = getattr(self, attr)
            if item is not None:
                new_item = self.remove_elements(item, to_be_removed)
                self.__setattr__(attr, new_item)

    @staticmethod
    def remove_elements(list_, indexes_to_be_removed):
        return [
            i for j, i in enumerate(list_) if j not in indexes_to_be_removed
        ]



# Scraper for Florida 4th District Court of Appeal Per Curiam Opinions
# CourtID: flaapp4pc
# Court Short Name: flaapp4pc

from juriscraper.opinions.united_states.state import fladistctapp_4_written


class Site(fladistctapp_4_written.Site):
    """Web Interface: http://www.4dca.org/opinions_auto/most_recent_pc.shtml"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.set_url("p")



# Scraper for the United States Court of Appeals for the Seventh Circuit
# CourtID: ca7
# Court Short Name: 7th Cir.

import feedparser

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://media.ca7.uscourts.gov/cgi-bin/OpinionsWeb/processWebInputExternal.pl?Time=month&startDate=&endDate=&Author=any&AuthorName=&Case=any&CaseYear=&CaseNum=&Rubmit=RssRecent&RssJudgeName=Sykes&OpsOnly=yes"
        self.court_id = self.__module__

    def _process_html(self):
        if self.test_mode_enabled():
            self.year = 2022
        feed = feedparser.parse(self.request["response"].content)
        for item in feed["entries"]:
            parts = item["summary"].split()
            docket = parts[parts.index("case#") + 1]
            name = item["summary"].split(docket)[1].split("(")[0]
            self.cases.append(
                {
                    "url": item["link"],
                    "docket": docket,
                    "date": item["published"],
                    "name": name,
                    "status": "Published",
                    "author": item["summary"].split("{")[1].split("}")[0],
                }
            )



# Author: William E. Palin
# Date created: 2023-01-04
# Notes: See Alabama

from juriscraper.opinions.united_states.state import ala


class Site(ala.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "https://judicial.alabama.gov/decision/civildecisions"



#!/usr/bin/env python
#
#  Takes an .html file on the command line, parses it using the PACER
#  Docket Report parser, and outputs json to stdout.

import sys

import jsondate3 as json

from juriscraper.pacer import DocketReport
from juriscraper.pacer.http import PacerSession

pacer_session = PacerSession(username="tr1234", password="Pass!234")
report = DocketReport("psc", pacer_session)

for path in sys.argv[1:]:
    with open(path) as f:
        report._parse_text(f.read())
    data = report.data
    print(json.dumps(data, indent=2, sort_keys=True, separators=(",", ": ")))



"""Scraper for Ninth Circuit of Appeals
CourtID: ca9
Court Short Name: ca9
"""

from juriscraper.lib.html_utils import get_row_column_text
from juriscraper.OralArgumentSiteLinear import OralArgumentSiteLinear


class Site(OralArgumentSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "https://www.ca9.uscourts.gov/media/"

    def _process_html(self):
        path = "//table[@id='search-results-table']//tr"
        rows = self.html.xpath(path)
        for row in rows[1:-2]:
            parts = row.xpath(".//td[6]/a/@href")[0].split("/")
            # the components needed to build the URL for the media file are
            # are all availble in this HTML element.  The path consisters of
            # year, month, day, and the docket number.mp3
            year = parts[-3][1:5]
            month = parts[-3][5:7]
            day = parts[-3][7:]
            docket_number = parts[-2]
            # Build URL for media file with the information we have
            # No need to traverse another page.
            url = f"https://cdn.ca9.uscourts.gov/datastore/media/{year}/{month}/{day}/{docket_number}.mp3"
            self.cases.append(
                {
                    "date": get_row_column_text(row, 5),
                    "docket": get_row_column_text(row, 2),
                    "judge": get_row_column_text(row, 3),
                    "name": get_row_column_text(row, 1),
                    "url": url,
                }
            )



# Scraper for Texas Criminal Court of Appeals
# CourtID: texcrimapp
# Court Short Name: TX
# Author: Michael Lissner
# Reviewer: None
# Date: 2015-09-02


from juriscraper.opinions.united_states.state import tex


class Site(tex.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court_name = "ccrimapp"
        self.checkbox = 1



from juriscraper.opinions.united_states.state import va


class Site(va.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.courts.state.va.us/wpcau.htm"
        self.court_id = self.__module__

    def _get_download_urls(self):
        path = "//p[./a[contains(./@href, '.pdf')]]/a[1]/@href"
        urls = [url for url in self.html.xpath(path) if url.endswith(".pdf")]
        return urls

    def _get_precedential_statuses(self):
        return ["Unpublished"] * len(self.case_names)



"""Scraper for Supreme Court of Northern Mariana Islands
CourtID: nmi
Court Short Name: NMI
Author: William Edward Palin
History:
  2023-01-21: Created by William Palin
"""
import re
from datetime import date
from typing import Any, Dict

from juriscraper.lib.string_utils import normalize_dashes
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        year = str(date.today().year)[-2:]
        self.url = f"https://www.cnmilaw.org/spm{year}.php#gsc.tab=0"
        self.status = "Published"

    def _cleanup_judge_names(self, judges: str) -> [str]:
        """Extract judge panel

        Because of a judge Torres,Jr. - and the various permutations of his
        Jr with and without commas and spacing we do a bit of cleanup to
        get his and the other judges corretly.  Additionally the author is
        sometimes denoted by a *.  This is cleaned up.

        :param judges: Content of the panel as a string
        :return: Judges as a string list
        """
        judges = [
            j.replace(" Jr.", "Jr.").strip(" *") for j in judges.split(",")
        ]
        judges = [j.replace("Jr.", " Jr.") for j in judges]
        return judges

    def _fetch_author(self, judges: str) -> str:
        """Parse the author from the judge text

        :param judges: Cell content
        :return: The author
        """
        if "*" not in judges:
            author = "Per Curiam"
        else:
            author = [j for j in judges.split(",") if "*" in j][0].strip("*")
        return author

    def _process_html(self):
        for s in self.html.xpath(".//td/a/@href[contains(., 'pdf')]/../../.."):
            cells = s.xpath(".//td")
            judge_text = cells[3].text_content()
            self.cases.append(
                {
                    "name": cells[0].text_content(),
                    "citation": cells[1].text_content(),
                    "date": cells[2].text_content(),
                    "judges": self._cleanup_judge_names(judge_text),
                    "author": self._fetch_author(judge_text),
                    "url": s.xpath(".//td/a/@href")[0],
                    "docket": "",
                }
            )

    def extract_from_text(self, scraped_text: str) -> Dict[str, Any]:
        """Pass scraped text into function and return data as a dictionary

        :param scraped_text: Text of scraped content
        :return: metadata containing the docket number
        """
        normalized_content = normalize_dashes(scraped_text)
        match = re.findall(r"Case No\.: (.*)", normalized_content)
        docket_number = match[0] if match else ""
        metadata = {
            "Docket": {
                "docket_number": docket_number,
            },
        }
        return metadata



# Editor: mlr
# Maintenance log
#    Date     | Issue
# 2013-01-28  | InsanityException due to the court adding busted share links.
# 2014-07-02  | New website required rewrite.

from datetime import datetime

from juriscraper.lib.string_utils import clean_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://media.ca11.uscourts.gov/opinions/pub/logname.php"
        self.back_scrape_iterable = list(range(20, 10000, 20))

    def _get_case_names(self):
        return [
            e for e in self.html.xpath("//tr[./td[1]/a//text()]/td[1]//text()")
        ]

    def _get_download_urls(self):
        return [
            e for e in self.html.xpath("//tr[./td[1]/a//text()]/td[1]/a/@href")
        ]

    def _get_case_dates(self):
        dates = []
        for date_string in self.html.xpath(
            "//tr[./td[1]/a//text()]/td[5]//text()"
        ):
            s = clean_string(date_string)
            if s == "00-00-0000" and "begin=21160" in self.url:
                # Bad data found during backscrape.
                s = "12-13-2006"
            dates.append(datetime.strptime(clean_string(s), "%m-%d-%Y").date())
        return dates

    def _get_docket_numbers(self):
        return [
            e for e in self.html.xpath("//tr[./td[1]/a//text()]/td[2]//text()")
        ]

    def _get_precedential_statuses(self):
        if "unpub" in self.url:
            return ["Unpublished"] * len(self.case_names)
        else:
            return ["Published"] * len(self.case_names)

    def _get_nature_of_suit(self):
        return [
            e for e in self.html.xpath("//tr[./td[1]/a//text()]/td[4]//text()")
        ]

    def _download_backwards(self, n):
        self.url = "http://media.ca11.uscourts.gov/opinions/pub/logname.php?begin={}&num={}&numBegin=1".format(
            n, n / 20 - 1
        )

        self.html = self._download()
        if self.html is not None:
            # Setting status is important because it prevents the download
            # function from being run a second time by the parse method.
            self.status = 200



"""Parse the mobile_query.pl ("Mobile Query" menu) result.

This provides the total number of docket entries, which is useful for alerts
"""
import pprint
import sys

from ..lib.log_tools import make_default_logger
from ..lib.utils import clean_court_object
from .docket_report import BaseDocketReport
from .reports import BaseReport

logger = make_default_logger()


class MobileQuery(BaseDocketReport, BaseReport):
    """Parse the mobile_query.pl ("Mobile Query" menu) result.

    This provides the total number of docket entries, which is useful for alerts
    """

    PATH = "cgi-bin/mobile_query.pl"

    CACHE_ATTRS = ["metadata"]

    def __init__(self, court_id, pacer_session=None):
        BaseDocketReport.__init__(self, court_id)
        BaseReport.__init__(self, court_id, pacer_session)

        # Initialize the empty cache properties
        self._clear_caches()
        self._metadata = None

    def parse(self):
        self._clear_caches()
        super().parse()

    @property
    def metadata(self):
        if self.is_valid is False:
            return {}

        if self._metadata is not None:
            return self._metadata

        # The data we're after look like this (re-spacing):
        # <a id="entriesLink" href="/cgi-bin/mobile_query.pl?search=dktEntry&caseid=186730&caseNum=4:06-cv-07294-PJH" data-transition="slide">
        #                             Docket Entries
        #                             <span class="ui-li-count">
        #                                 29
        #                             </span>
        #                         </a>
        # <div class="ui-title" style="padding-right: 90px; margin: .8em 0 1em 5%; font-size:14px; text-align: left">
        # Pages: 1,
        # Cost: 0.10

        ui_title = self.tree.xpath('.//div[@class="ui-title"]')[0]
        cost_raw = ui_title.text_content().strip()
        if (
            "Cost:" in cost_raw
        ):  # bad: means we were charged. Return immediately with bad news.
            cost = float(cost_raw.split("Cost: ", 1)[1])
            data = {"cost": cost, "cost_raw": cost_raw}
            self._metadata = data
            return data
        span = self.tree.xpath('.//a[@id="entriesLink"]//span')[0]
        entry_count = int(span.text_content().strip())
        data = {
            "court_id": self.court_id,
            "entry_count": entry_count,
            "cost": cost_raw,
        }

        data = clean_court_object(data)

        self._metadata = data
        return data

    def query(self, pacer_case_id):
        """Use a district court's PACER mobile query function with a known case id

        At the top of every district PACER court, there's a button that says,
        "Query". When you click that button, you can select "Mobile Query".
        It either does a query for the criteria you specify, returning a list
        of results or it returns the specific case you were looking for, including
        the number of docket entries in that case.

        While there is utility in both of these result types, this method only
        supports the use case where you know the pacer_case_id in advance, and
        are expecting only one result in return. This method does *not* support
        parsing the search results that the Query button can return. That use
        case is supposed by the CaseQueryAdvancedBankruptcy and
        CaseQueryAdvancedDistrict objects.

        :param pacer_case_id: A pacer_case_id for a case to lookup.
        :return None: Instead, sets self.response attribute and runs
        self.parse()
        """

        """
        This is an example of a query that could be sent, and which we're
        attempting to replicate:
        curl 'https://ecf.cand.uscourts.gov/cgi-bin/mobile_query.pl?search=caseInfo&caseid=186730' \
        -H 'Content-Type: application/x-www-form-urlencoded' \
        -H 'Referer: https://external'\
        -H 'Cookie: CASE_NUM=4:06-cv-07294(186730); PacerSession=xxx; NextGenCSO=xxx; PacerPref=receipt=Y' \

        The important notes, from testing:
         - The CASE_NUM cookie isn't needed.
         """
        assert (
            self.session is not None
        ), "session attribute of MobileQuery cannot be None."
        assert bool(
            pacer_case_id
        ), f"pacer_case_id must be truthy, not '{pacer_case_id}'"
        logger.info(
            "Running mobile query for case id '%s' in court '%s'",
            pacer_case_id,
            self.court_id,
        )
        self.response = self.session.post(
            f"{self.url}?search=caseInfo&caseid=={pacer_case_id}"
        )
        self.parse()

    @property
    def data(self):
        """Get all the data back from this endpoint.

        Don't attempt to return parties or docket_entries like the superclass
        does.
        """
        if self.is_valid is False:
            return {}

        data = self.metadata.copy()
        return data


def _main():
    if len(sys.argv) != 2:
        print("Usage: python -m juriscraper.pacer.mobile_query filepath")
        print("Please provide a path to an HTML file to parse.")
        sys.exit(1)

    report = MobileQuery("mad")
    filepath = sys.argv[1]
    print(f"Parsing HTML file at {filepath}")
    with open(filepath) as f:
        text = f.read().decode("utf-8")
    report._parse_text(text)
    pprint.pprint(report.data, indent=2)


if __name__ == "__main__":
    _main()



"""Scraper for the Rhode Island Supreme Court
CourtID: ri
Court Short Name: R.I.
Court Contact: (see ri_p)
Author: Brian W. Carver
History:
    Date created: 2013-08-10 by Brian W. Carver
    2022-05-02: Updated by flooie, to use JSON responses
"""

from juriscraper.opinions.united_states.state import ri_p


class Site(ri_p.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Unpublished"
        self.opinion_type = "Orders"
        self.url = self.build_url()



from datetime import date, timedelta

from dateutil.rrule import MONTHLY, rrule

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.start = date.today() - timedelta(days=60)
        self.end = date.today()
        self.url = "http://www.courts.wa.gov/opinions/index.cfm?fa=opinions.processSearch"
        self.method = "POST"

        self.courtLevel = "S"
        self.pubStatus = "All"
        self._set_parameters()
        self.base = "//tr[../tr/td/strong[contains(., 'File Date')]]"
        self.back_scrape_iterable = [
            i.date()
            for i in rrule(
                MONTHLY,
                dtstart=date(2014, 2, 1),
                until=date.today(),
            )
        ]

    def _set_parameters(self):
        self.parameters = {
            "courtLevel": self.courtLevel,
            "pubStatus": self.pubStatus,
            "beginDate": self.start.strftime("%m/%d/%Y"),
            "endDate": self.end.strftime("%m/%d/%Y"),
            "SType": "Phrase",
            "SValue": "",
        }

    def _get_case_names(self):
        path = f"{self.base}/td[3]/text()"
        return list(self.html.xpath(path))

    def _get_docket_numbers(self):
        path = f"{self.base}/td[2]/a/text()"
        return list(self.html.xpath(path))

    def _get_case_dates(self):
        path = f"{self.base}/td[1]/text()"
        return [
            convert_date_string(date.strip())
            for date in self.html.xpath(path)
            if date.strip()
        ]

    def _get_download_urls(self):
        path = f"{self.base}/td[2]/a[2]/@href"
        return list(self.html.xpath(path))

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _download_backwards(self, d):
        self.start = d
        self.end = d + timedelta(days=32)
        self._set_parameters()
        self.html = self._download()



"""Scraper for Minnesota Attorney General Opinions
CourtID: minnag
Court Short Name: MN
Author: David Cook
Reviewer:
"""

import re

from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.domain = "http://www.ag.state.mn.us"
        self.url = "http://www.ag.state.mn.us/office/opinions/DATE.asp"
        self.opinions = []

    def _download(self, request_dict={}):
        html = super()._download(request_dict)
        self._extract_case_data_from_html(html)
        return html

    def _extract_case_data_from_html(self, html):
        for p in html.xpath("//div[@id='content']/div[@id='op']/p"):
            text = p.xpath("a/strong/following-sibling::text()[1]")[0]
            name = " ".join(
                text.split()[1:]
            )  # strip leading dash charater, which can be one of various unicode dash characters
            name = re.sub("\\s+", " ", name).strip()
            summary = " ".join(
                p.xpath(
                    "br/following-sibling::text() | br/following-sibling::*//text()"
                )
            )
            summary = re.sub("\\s+", " ", summary).strip()

            self.opinions.append(
                {
                    "name": name,
                    "url": p.xpath("a/@href")[0],
                    "date": convert_date_string(p.xpath("a/strong/text()")[0]),
                    "summary": summary,
                }
            )

    def _get_case_names(self):
        return [opinion["name"] for opinion in self.opinions]

    def _get_download_urls(self):
        return [opinion["url"] for opinion in self.opinions]

    def _get_case_dates(self):
        return [opinion["date"] for opinion in self.opinions]

    def _get_summaries(self):
        return [opinion["summary"] for opinion in self.opinions]

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.opinions)



from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OralArgumentSite import OralArgumentSite


class OralArgumentSiteLinear(OralArgumentSite):
    """This class can be used for any site that needs to be scraped linearly,
    instead of, for example, with separate html path parsing getters. Sometimes
    it is just easier and less repetitive to scrape a site this way, in which
    case you can simply extend this class and implement _process_html().
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cases = []
        self.status = None

    def _process_html(self):
        raise Exception(
            "Must implement _process_html() on OralArgumentSiteLinear child"
        )

    def _get_case_names(self):
        return [case["name"] for case in self.cases]

    def _get_download_urls(self):
        return [case["url"] for case in self.cases]

    def _get_case_dates(self):
        return [convert_date_string(case["date"]) for case in self.cases]

    def _get_docket_numbers(self):
        return [case["docket"] for case in self.cases]

    # optional getters below

    def _get_optional_field_by_id(self, id):
        if self.cases and id in self.cases[0]:
            return [case[id] for case in self.cases]

    def _get_judges(self):
        return self._get_optional_field_by_id("judge")



# Scraper for the United States Tax Court
# CourtID: tax
# Court Short Name: Tax Ct.
# Neutral Citation Format (Tax Court opinions): 138 T.C. No. 1 (2012)
# Neutral Citation Format (Memorandum opinions): T.C. Memo 2012-1
# Neutral Citation Format (Summary opinions: T.C. Summary Opinion 2012-1
import json
from datetime import date, timedelta

from juriscraper.lib.string_utils import titlecase
from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_blue_green = None
        self.base = "https://public-api-green.dawson.ustaxcourt.gov/public-api"
        self.url = f"{self.base}/opinion-search"
        self.court_id = self.__module__
        self.td = date.today()
        today = self.td.strftime("%m/%d/%Y")
        self.method = "GET"

        last_month = (self.td - timedelta(days=31)).strftime("%m/%d/%Y")
        self.params = {
            "dateRange": "customDates",
            "startDate": last_month,
            "endDate": today,
            "opinionTypes": "MOP,SOP,TCOP",
        }

    def _download(self, request_dict={}):
        """Download from api

        The tax court switches between blue and green deploys so we need to
        check which one is current before we continue

        :param request_dict: An empty dictionary.
        :return: None
        """
        if not self.set_blue_green and not self.test_mode_enabled():
            check = self.request["session"].get(self.url)
            if check.status_code != 200:
                self.base = (
                    "https://public-api-blue.dawson.ustaxcourt.gov/public-api"
                )
                self.url = f"{self.base}/opinion-search"
            self.set_blue_green = True
        if self.test_mode_enabled():
            with open(self.url) as file:
                self.json = json.load(file)
        else:
            self.json = (
                self.request["session"]
                .get(
                    self.url,
                    params=self.params,
                )
                .json()
            )

    def _process_html(self) -> None:
        """Process the html

        Iterate over each item on the page collecting our data.
        return: None
        """
        for case in self.json:
            url = self._get_url(case["docketNumber"], case["docketEntryId"])
            status = (
                "Published"
                if case["documentType"] == "T.C. Opinion"
                else "Unpublished"
            )
            self.cases.append(
                {
                    "judge": case.get("signedJudgeName", ""),
                    "date": case["filingDate"][:10],
                    "docket": case["docketNumber"],
                    "url": url,
                    "name": titlecase(case["caseCaption"]),
                    "status": status,
                }
            )

    def _get_url(self, docket_number: str, docketEntryId: str) -> str:
        """Fetch the PDF URL with AWS API key

        param docket_number: The docket number
        param docketEntryId: The docket entry id
        return: The URL to the PDF
        """
        self.url = f"{self.base}/{docket_number}/{docketEntryId}/public-document-download-url"
        if self.test_mode_enabled():
            # Don't fetch urls when running tests.  Because it requires
            # a second api request.
            return self.url
        pdf_url = super()._download()["url"]
        return pdf_url



# Scraper for Minnesota Court of Appeals Published Opinions
# CourtID: minnctapp
# Court Short Name: MN
# Author: mlr
# Date: 2016-06-03


from juriscraper.opinions.united_states.state import minn


class Site(minn.Site):
    # Only subclasses minn for the _download method.
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.court_filters = ["/ctapun/", "/ctappub/"]



"""Scraper for the United States Court of Federal Claims
CourtID: uscfc
Court Short Name: Fed. Cl.

Notes:
    Scraper adapted for new website as of February 20, 2014.
"""

import datetime
import re

from lxml import html

from juriscraper.lib.exceptions import InsanityException
from juriscraper.lib.string_utils import (
    clean_if_py3,
    convert_date_string,
    titlecase,
)
from juriscraper.OpinionSite import OpinionSite


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.url = "http://www.uscfc.uscourts.gov/aggregator/sources/8"
        self.back_scrape_iterable = list(range(1, 4))
        self.court_id = self.__module__
        self.today = datetime.datetime.now()

    def _download(self, request_dict={}):
        if self.test_mode_enabled():
            # Use static 'today' date for consisting test results
            self.today = convert_date_string("2018/10/17")
        return super()._download(request_dict)

    def _get_case_dates(self):
        dates = []
        for item in self.html.xpath('//span[@class="feed-item-date"]'):
            text = item.text_content().strip()
            words = text.split()
            if len(words) == 2:
                date = convert_date_string(words[1])
            elif "ago" in text:
                # The record was added today "X hours and Y min ago"
                date = self.today
            else:
                raise InsanityException(
                    f"Unrecognized date element string: {text}"
                )
            dates.append(date)
        return dates

    def _get_case_names(self):
        case_names = []
        for t in self.html.xpath('//h3[@class="feed-item-title"]//text()'):
            t = " ".join(clean_if_py3(t).split())  # Normalize whitespace
            if t.strip():
                # If there is something other than whitespace...
                if not isinstance(t, str):
                    t = str(t, encoding="utf-8")

                if " • " in t:
                    t = t.split(" • ")[1].strip()
                t = titlecase(t.lower())
                case_names.append(t)
        return case_names

    def _get_download_urls(self):
        path = '//h3[@class="feed-item-title"]/a/@href'
        return list(self.html.xpath(path))

    def _get_precedential_statuses(self):
        return ["Published"] * len(self.case_names)

    def _get_docket_numbers(self):
        docket_numbers = []
        for t in self.html.xpath('//h3[@class="feed-item-title"]//text()'):
            t = clean_if_py3(t)
            if t.strip():
                # If there is something other than whitespace...
                if not isinstance(t, str):
                    t = str(t, encoding="utf-8")

                if " • " in t:
                    t = t.split(" • ")[0].strip()
                docket_numbers.append(t)
        return docket_numbers

    def _get_summaries(self):
        summaries = []
        path = '//div[@class="feed-item-body"]'
        for e in self.html.xpath(path):
            s = html.tostring(e, method="text", encoding="unicode")
            s = clean_if_py3(s).split("Keywords:")[0]
            summaries.append(s)

        return summaries

    def _get_judges(self):
        path = '//div[@class="feed-item-body"]'
        judges = []
        splitters = [
            "Signed by Chief Judge",
            "Signed by Judge",
            "Signed by Chief Special Master",  # Vaccine courts have odd names for judges
            "Signed by Special Master",
        ]
        for e in self.html.xpath(path):
            t = html.tostring(e, method="text", encoding="unicode")
            t = clean_if_py3(t).split("Keywords:")[0]
            for splitter in splitters:
                judge_parts = t.rsplit(splitter)
                if len(judge_parts) == 1:
                    # No splits found...
                    judge = ""
                    continue
                else:
                    judge = judge_parts[1]
                    break

            # Often the text looks like: 'Judge Susan G. Braden. (jt1) Copy to parties.' In that case we only
            # want the name, not the rest.
            length_of_match = 2
            m = re.search(
                r"[a-z]{%s}\." % length_of_match, judge
            )  # Two lower case letters followed by a period
            if m:
                judge = judge[: m.start() + length_of_match]
            else:
                judge = ""
            judge.strip(".")
            judges.append(judge)
        return judges

    def _download_backwards(self, page):
        self.url = (
            f"http://www.uscfc.uscourts.gov/aggregator/sources/8?page={page}"
        )
        self.html = self._download()



"""Scraper for Supreme Court of Guam
CourtID: guam
Court Short Name: Guam
Author: mmantel
History:
  2019-12-09: Created by mmantel
"""

import re
from datetime import date

from juriscraper.OpinionSiteLinear import OpinionSiteLinear


class Site(OpinionSiteLinear):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = "http://www.guamsupremecourt.com/Supreme-Court-Opinions/Supreme-Court-Opinions.asp"
        self._year = date.today().year
        self.parameters = {"Year": str(self._year)}
        self.method = "POST"
        self.status = "Published"

        # The year dropdown goes back to 1990, but the Court wasn't
        # created until 1996 and there are no opinions posted for
        # prior years.
        self.back_scrape_iterable = range(1996, self._year)

    def _process_html(self):
        year = 2019 if self.test_mode_enabled() else self._year
        middle_of_year = "July 2, %d" % year

        for table in self.html.xpath(
            '//a[@id="Opinion"]/following-sibling::table'
        ):
            text = table.xpath(".//td/text()")[0]
            # Text is typically something like this:
            # CVA16-016, 2018 Guam 8, July 26, 2018
            # But formatting varies from year to year, and some elements
            # are not provided for some cases.

            date_match = re.search(r"[A-Za-z]+\.?\s+[0-9]+,\s+[0-9]+", text)
            docket_match = re.search(r"[A-Z]+[0-9]+\-+[0-9]+", text)
            neutral_citation_match = re.search(r"[0-9]+\s+Guam\s+[0-9]+", text)

            self.cases.append(
                {
                    "date": date_match.group(0)
                    if date_match
                    else middle_of_year,
                    "date_filed_is_approximate": date_match is None,
                    "docket": docket_match.group(0) if docket_match else "",
                    "name": table.xpath(".//a/text()")[0],
                    "citation": neutral_citation_match.group(0)
                    if neutral_citation_match
                    else "",
                    "url": table.xpath(".//a/@href")[0],
                }
            )

    def _download_backwards(self, year):
        self._year = year
        self.parameters = {"Year": str(year)}
        self.html = self._download()
        self._process_html()



from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OpinionSite import OpinionSite


class OpinionSiteLinear(OpinionSite):
    """This class can be used for any site that needs to be scraped linearly,
    instead of, for example, with separate html path parsing getters. Sometimes
    it is just easier and less repetitive to scrape a site this way, in which
    case you can simply extend this class and implement _process_html().
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cases = []
        self.status = None

    def _process_html(self):
        raise Exception(
            "Must implement _process_html() on OpinionSiteLinear child"
        )

    def _get_case_names(self):
        return [case["name"] for case in self.cases]

    def _get_download_urls(self):
        return [case["url"] for case in self.cases]

    def _get_case_dates(self):
        return [convert_date_string(case["date"]) for case in self.cases]

    def _get_date_filed_is_approximate(self):
        return [
            case.get("date_filed_is_approximate", False) for case in self.cases
        ]

    def _get_precedential_statuses(self):
        # first try to use status values set in cases dictionary
        try:
            return [case["status"] for case in self.cases]
        except AttributeError:
            pass
        except KeyError:
            pass
        # we fall back on using singular status defined in init,
        # which is all you need to do if all cases on the page
        # have the same status
        if not self.status:
            raise Exception(
                "Must define self.status in __init__ on OpinionSiteLinear child"
            )
        return [self.status] * len(self.cases)

    def _get_docket_numbers(self):
        return [case["docket"] for case in self.cases]

    # optional getters below

    def _get_optional_field_by_id(self, id):
        if self.cases and id in self.cases[0]:
            return [case[id] for case in self.cases]

    def _get_judges(self):
        return self._get_optional_field_by_id("judge")

    def _get_citations(self):
        return self._get_optional_field_by_id("citation")

    def _get_parallel_citations(self):
        return self._get_optional_field_by_id("parallel_citation")

    def _get_summaries(self):
        return self._get_optional_field_by_id("summary")

    def _get_lower_courts(self):
        return self._get_optional_field_by_id("lower_court")



"""
Scraper for Alabama Supreme Court
CourtID: ala
Court Short Name: Alabama
Author: William Palin
Court Contact:
History:
 - 2023-01-04: Created.
Notes:
 - This may be the only court (with the app. cts) that requires selenium
"""

import re

from lxml.html import fromstring

from juriscraper.OpinionSiteLinearWebDriven import OpinionSiteLinearWebDriven


class Site(OpinionSiteLinearWebDriven):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = (
            "https://judicial.alabama.gov/decision/supremecourtdecisions"
        )
        self.uses_selenium = True

    def _process_html(self):
        if not self.test_mode_enabled():
            self.initiate_webdriven_session()

        # Fetch from the last four Released documents
        if not self.test_mode_enabled():
            links = self.html.xpath('//*[@id="FileTable"]/tbody/tr/td/a')[:4]
        else:
            links = ["fake_link"]
        for link in links:
            if not self.test_mode_enabled():
                # Fetch the PDF at the a_href link in the file table and parse it
                self.webdriver.get(link.get("href"))
                date_filed = link.text_content().split("day, ")[1].strip()
                # Wait for pdf to render
                self.webdriver.implicitly_wait(10000)
                # And click next twice to make sure we fully render the PDF content
                self.find_element_by_id("next").click()
                self.find_element_by_id("next").click()

                # Set the content for lxml parsing after this rendering.
                self.html = fromstring(self.webdriver.page_source)
            else:
                super()._download()
                date_filed = "JANUARY 4, 2023"

            # Iterate over the elements to extract out the content
            start = False
            content = []
            author = ""
            for element in self.html.xpath("//*"):
                left = element.xpath(
                    'substring-before(substring-after(@style, "left: "), "px")'
                )
                if element.cssselect("span[class='markedContent']"):
                    continue
                elif left and round(float(left)) == 120:
                    start = True
                    content = []
                    content.append(element.text_content())

                    text = element.text_content()
                    if (
                        "JUDGE" in text
                        or "JUSTICE" in text
                        or "PER CURIAM" in text
                    ):
                        author = text
                    if "REHEARING" in text:
                        author = ""

                elif start and element.cssselect("span[aria-owns]"):
                    ao = element.get("aria-owns")
                    if not ao:
                        start = False
                        continue
                    urls = self.html.xpath(
                        f".//a[@id='{ao.split()[0]}']/@href"
                    )
                    # Cleanup whitespace and identify docket and case title
                    content.append(element.text_content())
                    all_content = " ".join(content)
                    cleaned_content = re.sub(r"\s{2,}", " ", all_content)
                    docket = re.findall(r"^[\w-]*", cleaned_content)[0]
                    title = (
                        cleaned_content.split("(")[0]
                        .replace(docket, "")
                        .strip()
                    )
                    content = []
                    start = False
                    # Add case and start over
                    self.cases.append(
                        {
                            "date": date_filed,
                            "name": title,
                            "docket": docket,
                            "status": "Published",
                            "url": urls[0],
                            "judge": author,
                        }
                    )
                elif start:
                    content.append(element.text_content())
        if not self.test_mode_enabled():
            self.webdriver.quit()



"""Scraper for Tenth Circuit of Appeals
CourtID: ca10
Author: mlr
"""

import datetime

from juriscraper.lib.html_utils import (
    get_table_column_links,
    get_table_column_text,
)
from juriscraper.lib.string_utils import convert_date_string
from juriscraper.OralArgumentSite import OralArgumentSite


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        # query for results of current month
        now = datetime.datetime.now()
        start_date = now - datetime.timedelta(weeks=4)
        start = "%d/%d/%d" % (
            start_date.month,
            start_date.day,
            start_date.year,
        )
        end = "%d/%d/%d" % (now.month, now.day, now.year)
        self.url = (
            "https://www.ca10.uscourts.gov/oralargument/search-results?"
            "field_oa_hearing_date_value[min][date]=%s&"
            "field_oa_hearing_date_value[max][date]=%s"
        ) % (start, end)

    def _get_download_urls(self):
        return get_table_column_links(self.html, 6)

    def _get_case_names(self):
        return get_table_column_text(self.html, 3)

    def _get_docket_numbers(self):
        dockets = get_table_column_text(self.html, 2)
        # Handle case where "no results" message is put in table cell, see: ca10_example_2.html
        return (
            []
            if len(dockets) == 1 and "no results" in dockets[0].lower()
            else dockets
        )

    def _get_case_dates(self):
        return [
            convert_date_string(ds)
            for ds in get_table_column_text(self.html, 4)
        ]



"""Scraper for Connecticut Appellate Court
CourtID: connctapp
Court Short Name: Connappct.
Author: Asadullah Baig<asadullahbeg@outlook.com>
Date created: 2014-07-11
History:
    - 2022-02-02, satsuki-chan: Fixed docket and name separator, changed super class to OpinionSiteLinear
"""

from juriscraper.opinions.united_states.state import conn


class Site(conn.Site):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = f"http://www.jud.ct.gov/external/supapp/archiveAROap{self.year}.htm"
        self.docket_regex = r"AC\d+"



from requests.cookies import RequestsCookieJar


def normalize_cookies(cookies):
    """Takes cookies from Selenium or from Python Requests and
    converts them to dict.

    This throws away information that Selenium otherwise has (like the host and
    such), but a dict is essentially all we need.
    """
    requests_cookies = {}
    if type(cookies) == list:
        # Selenium cookies
        for cookie in cookies:
            requests_cookies[cookie["name"]] = cookie["value"]
    elif type(cookies) == RequestsCookieJar:
        # Requests cookies. Convert to dict.
        requests_cookies = dict(cookies)

    return requests_cookies
